{******************************************************************************}
{                                                                              }
{ ETWTSQLProfiler                                                              }
{                                                                              }
{ Copyright (C) 2008 Franck SORIANO                                            }
{                                                                              }
{ This program is free software; you can redistribute it and/or                }
{  modify it under the terms of the GNU General Public License                 }
{ as published by the Free Software Foundation; either version 2               }
{ of the License, or (at your option) any later version.                       }
{                                                                              }
{ This program is distributed in the hope that it will be useful,              }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of               }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                }
{ GNU General Public License for more details.                                 }
{                                                                              }
{ You should have received a copy of the GNU General Public License            }
{ along with this program; if not, write to the Free Software                  }
{ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,MA  02110-1301,USA.}
{                                                                              }
{******************************************************************************}
unit uMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, Grids, StdCtrls, ComCtrls, ImgList, ActnList, ToolWin, XPMan,
  uETW, uETWClient, uETWTraceControler;

type
  TfrmMain = class(TForm)
    ToolBar1: TToolBar;
    acList: TActionList;
    sb: TStatusBar;
    Panel1: TPanel;
    grd: TDrawGrid;
    Splitter1: TSplitter;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    acStart: TAction;
    acStop: TAction;
    acClear: TAction;
    TmRefresh: TTimer;
    XPManifest1: TXPManifest;
    imgList: TImageList;
    lblSQL: TLabel;
    eSQL: TMemo;
    procedure grdClick(Sender: TObject);
    procedure grdSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure grdDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect;
      State: TGridDrawState);
    procedure TmRefreshTimer(Sender: TObject);
    procedure acClearExecute(Sender: TObject);
    procedure acStopExecute(Sender: TObject);
    procedure acStartExecute(Sender: TObject);
    procedure acStopUpdate(Sender: TObject);
    procedure acStartUpdate(Sender: TObject);
  private
    FTraceControler : TETWTraceControler; // contrleur de la trace.
    FTrace : TETWClient; // Lit la trace gnre.

    procedure UpdateTrace; // Rafraichit l'affichage de la trace.
    procedure UpdateStatusBar;  // Rafraichit l'affichage de la base d'tat.

  protected
    procedure DoCreate; override;
    procedure DoDestroy; override;
  public

  end;

var
  frmMain: TfrmMain;

implementation
uses uLogger, uSQLLogger, uEventTrace;
const
  TraceSessionName = 'ETWSQLProfiler';
{$R *.dfm}

const
  COL_LIGNE = 0;
  COL_EVENTCLASS = 1;
  COL_EVENTTYPE = 2;
  COL_DELTA = 3;
  COL_USERDATA = 4;
  
// Efface les vnements de la trace en cours
procedure TfrmMain.acClearExecute(Sender: TObject);
begin
  // Efface les donnes de la trace.
  FTrace.ClearTrace;

  // On rafraichit l'affichage.
  UpdateTrace;
  UpdateStatusBar;
end;

// Dmarre une nouvelle trace temps rel.
procedure TfrmMain.acStartExecute(Sender: TObject);
begin
  // Si une trace tait dj dmarre, on commence par la dtruire.
  if Assigned(FTrace)
  then begin
    FTrace.Free;
    FTrace := nil;
  end;
  
  // Crer et dmarre une trace temps rel.
  FTraceControler := TETWTraceControler.Create(TraceSessionName, '');

  // On active les deux providers DefaultLogger et SQLLogger.
  FTraceControler.EnableProvider(DefaultLogger, TRACE_LEVEL_VERBOSE, 0);
  FTraceControler.EnableProvider(SQLLoggerGUID, TRACE_LEVEL_VERBOSE, 0);

  // Enfin, on connecte le client sur la trace temps rel qu'on vient de crer
  try
    FTrace := TETWClient.Create(TraceSessionName, true);
    TmRefresh.Enabled := true; // On active le timer pour rafraichir la trace.
  except
    // En cas d'erreur sur l'initialisation du client, on arrte le contrleur
    Application.HandleException(self); // Affichage de l'erreur.
    FTraceControler.Free;   // Arrt du contrleur.
    FTraceControler := nil;
  end;
end;

procedure TfrmMain.acStartUpdate(Sender: TObject);
begin
  // On ne peut dmarrer une trace que si elle n'est pas dj en cours
  // d'excution.
  TAction(Sender).Enabled := not Assigned(FTraceControler);
end;

procedure TfrmMain.acStopExecute(Sender: TObject);
begin
  // 0n dtruit le controler pour arrter la trace.
  FTraceControler.Free;
  FTraceControler := nil;

  // Par contre, on garde le client pour pouvoir continuer  afficher la trace.
  TmRefresh.Enabled := false; // Il n'est plus ncessaire de rafraichir la trace
end;

procedure TfrmMain.acStopUpdate(Sender: TObject);
begin
  // On ne peut arrter la trace que si elle a t dmarre.
  TAction(Sender).Enabled := Assigned(FTraceControler);
end;

procedure TfrmMain.DoCreate;
begin
  inherited DoCreate;

  // Initialise la grille d'affichage des vnements.
  grd.ColCount := 5;
  grd.ColWidths[4] := 1000;
end;

procedure TfrmMain.DoDestroy;
begin
  inherited DoDestroy;

  // On arrte la trace si elle est toujours en cours d'excution.
  // Attention, la trace s'execute dans ETW. Si on ne demande pas  ETW
  // d'arrter la trace, elle reste en cours d'excution mme aprs la fermeture
  // de l'application.
  if Assigned(FTraceControler)
  then FTraceControler.Free;

  if Assigned(FTrace)
  then FTrace.Free;
end;

procedure TfrmMain.grdClick(Sender: TObject);
begin
  // Si on click sur une cellule, on dplace ou modifie la slection. Il faut
  // donc mettre  jour l'affichage de la barre d'tat.
  UpdateStatusBar;
end;

// gradDrawCell doit dessiner le contenu de chaque cellule de la grille. C'est
// donc elle qui s'occupe d'affciher la trace  l'cran.
procedure TfrmMain.grdDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
const EventColor : array [0..10]of TColor = (clBlack, clGray, clBlack, clRed,
   clFuchsia, clFuchsia, clFuchsia, clMaroon, clMaroon, clGreen, clGreen);
var
  Event : TEventTrace;
  t0 : int64;

  // Fonction utilitaire pour crire dans la grille
  procedure DrawText(Rect: TRect; txt: string; Fmt: TTextFormat);
  begin
    grd.Canvas.TextRect(Rect, txt, fmt);
  end;

begin
  // On rduit la zone de dessin pour ne pas crire sur les bords de la cellule.
  inc(rect.Top);
  inc(rect.left, 2);
  dec(rect.Right, 2);
  dec(rect.Bottom);

  if ARow = 0 // S'agit-il de la ligne de titre ? 
  then begin
    // On dessine les intituls des colonnes.
    case ACol of
    COL_LIGNE : DrawText(Rect, 'Ligne', [tfCenter]);
    COL_EVENTCLASS : DrawText(Rect, 'Class', [tfCenter]);
    COL_EVENTTYPE : DrawText(Rect, 'Type', [tfCenter]);
    COL_DELTA : DrawText(Rect, 'Delta', [tfCenter]);
    COL_USERDATA : DrawText(Rect, 'Text', [tfCenter]);
    end;
  end
  else begin
    // Sinon, on dessine les donnes de la trace.
    if Assigned(FTrace) and (ARow-1<FTrace.Trace.Count)
    then begin
      // La ligne a dessiner fait partie de la trace.

      // Pour la colonne SQL on dessine la requte avec la police Courier New
      // pour les autres colonne, on garde la police standard.
      if ACol = COL_USERDATA
      then grd.Canvas.Font := lblSQL.Font
      else grd.Canvas.Font := grd.Font;

      // On lit l'vnement  dessiner.
      Event := TEventTrace(FTrace.Trace[ARow-1]);

      // On lit la date de l'vnement prcdent pour calculer le temps coul
      // entre les deux. Pour le premier vnement on prend la date de
      // l'vnement courant.
      if ARow-2 >=0
      then t0 := TEventTrace(FTrace.Trace[ARow-2]).TimeStamp
      else t0 := Event.TimeStamp;

      // Si la ligne fait partie de la slection, on force la couleur du texte
      //  clWhite. Sinon la ligne devient illisible selon les paramtrages des
      // couleurs systme.
      if gdSelected in State
      then grd.Canvas.Font.Color := clWhite
      else begin
        // La ligne n'est pas slection. La couleur de dessin dpend du type
        // de l'vnement. Cf tableau EventColor.
        // A dfaut, l'vnement est dessin en noir.
        if Event.EventType <= high(EventColor)
        then grd.Canvas.Font.Color := EventColor[Event.EventType]
        else grd.Canvas.Font.Color := clBlack;
      end;

      // Enfin, il ne reste plus qu' faire le dessin proprement dit.
      case ACol of
      COL_LIGNE: // On dessine le numro de ligne.
        DrawText(Rect, IntToStr(ARow-1), [tfRight]);
      COL_EVENTCLASS: // On dessine le nom de l'vnement.
        DrawText(Rect, Event.GetEventClass, [tfCenter]);
      COL_EVENTTYPE : // On dessine le nom de l'EventType
        DrawText(Rect, Event.GetEventType, [tfCenter]);
      COL_DELTA: // On affiche la dure coule depuis l'vnemenet prcdent, en ms.
        DrawText(Rect, Format('%.4f', [(Event.TimeStamp-t0)/10000]), [tfRight]);
      COL_USERDATA: // Enfin on affiche les donnes utilisateur de l'vnement, c'est 
          // dire, la requte SQL.
        DrawText(Rect, Event.MofFieldToString, [tfSingleLine]);
      else
        DrawText(Rect, '', []); // Par dfaut, on efface la cellule.
      end;
    end
    else DrawText(Rect, '', []); // La ligne ne fait pas partie de la trace,
                                 // on efface la cellule.
  end;
end;

procedure TfrmMain.grdSelectCell(Sender: TObject; ACol, ARow: Integer;
  var CanSelect: Boolean);
var Event : TSQLEvent;  
begin
  // Lorqu'on slectionne un vnement et que ce dernier correspond  une
  // requte SQL, on affiche la requte SQL dans SynEdit en dessous de faon
  //  pouvoir la voir dans sa totalit, avec la coloration syntaxique.
  CanSelect := true;
  if Assigned(FTrace) and (ARow-1<FTrace.Trace.Count)
  then begin
    Event := TSQLEvent(FTrace.Trace[ARow-1]);
    if (Event.EventType = EVENT_SQL_FORMAT_END) or
       (Event.EventType = EVENT_SQL_END)
    then eSQL.Lines.Text := Event.Msg
    else eSQL.Lines.Text := '';
  end;
end;

procedure TfrmMain.TmRefreshTimer(Sender: TObject);
begin
  if Assigned(FTraceControler)
  then begin
    // Force l'criture des buffers dans la trace.
    FTraceControler.Flush;

    // Puis on met  jour l'affichage.
    UpdateTrace;
    UpdateStatusBar;
  end;
end;

procedure TfrmMain.UpdateStatusBar;
var
  i : integer;
  SelFrom, SelTo : integer;
  Clock0, Clock1 : int64;
begin
  // La barre d'tat n'affiche rien s'il n'y a pas de trace.
  if Assigned(FTrace)
  then begin
    // Premirement on rcupre la slection. 
    SelFrom := grd.Selection.Top -1;
    SelTo := grd.Selection.Bottom -1;

    // On cherche la date du premier vnement slectionn
    if SelFrom<FTrace.Trace.Count
    then Clock0 := TEventTrace(FTrace.Trace[SelFrom]).TimeStamp
    else Clock0 := 0;

    // On cherche la date du dernier vnement slectionn
    if SelTo<FTrace.Trace.Count
    then Clock1 := TEventTrace(FTrace.Trace[SelTo]).TimeStamp
    else Clock1 := 0;

    // Enfin, il ne resteplus qu' afficher les informations

    // Lignes slectionnes.
    sb.Panels[0].Text := Format('Selection : %d - %d', [SelFrom, SelTo]);
    sb.Panels[1].Text := IntToStr(Clock0); // Date du premier vnement
    sb.Panels[2].Text := IntToStr(Clock1); // Date du deuxime vnement
    sb.Panels[3].Text := Format('Dure : %.4f ms', [(Clock1-Clock0) / 10000]);
  end
  else begin
    for i := 0 to sb.Panels.Count-1 do
    begin
      sb.Panels[i].Text := '';
    end;
  end;  
end;

procedure TfrmMain.UpdateTrace;
begin
  // Pour mettre  jour la trace, il suffit de redfinir le nombre de ligne dans
  // la grille. Ca va provoquer un raffichage de la grille et les cellules
  // seront redessines.
  if Assigned(FTrace)
  then grd.RowCount := FTrace.Trace.Count +2
  else grd.RowCount := 2;
end;

end.
