{******************************************************************************}
{                                                                              }
{ ETWTraceViewer                                                               }
{                                                                              }
{ Copyright (C) 2008 Franck SORIANO                                            }
{                                                                              }
{ This program is free software; you can redistribute it and/or                }
{  modify it under the terms of the GNU General Public License                 }
{ as published by the Free Software Foundation; either version 2               }
{ of the License, or (at your option) any later version.                       }
{                                                                              }
{ This program is distributed in the hope that it will be useful,              }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of               }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                }
{ GNU General Public License for more details.                                 }
{                                                                              }
{ You should have received a copy of the GNU General Public License            }
{ along with this program; if not, write to the Free Software                  }
{ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,MA  02110-1301,USA.}
{                                                                              }
{******************************************************************************}
unit uCreateTrace;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, Buttons, ExtCtrls, ActnList, Menus;

type
  // Boite de dialogue pour dfinir les paramtres ncessaires  la cration
  // d'une nouvelle trace.
  TfrmCreateTrace = class(TForm)
    pnlTop: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    eTraceName: TEdit;
    gbTraceType: TGroupBox;
    rdbtRT: TRadioButton;
    rdbtFile: TRadioButton;
    eFileName: TEdit;
    btBrowse: TSpeedButton;
    lblTraceFile: TLabel;
    gbProviders: TGroupBox;
    lvProviders: TListView;
    btOk: TButton;
    btCancel: TButton;
    ActionList1: TActionList;
    acBrowse: TAction;
    acCreate: TAction;
    acCancel: TAction;
    opDlg: TOpenDialog;
    Label3: TLabel;
    cbTraceLevel: TComboBox;
    PopupMenu1: TPopupMenu;
    Dfinirlesflagsdactivation1: TMenuItem;
    procedure Dfinirlesflagsdactivation1Click(Sender: TObject);
    procedure acCancelExecute(Sender: TObject);
    procedure acCreateExecute(Sender: TObject);
    procedure acCreateUpdate(Sender: TObject);
    procedure lvProvidersCompare(Sender: TObject; Item1, Item2: TListItem;
      Data: Integer; var Compare: Integer);
    procedure acBrowseExecute(Sender: TObject);
    procedure acBrowseUpdate(Sender: TObject);
  private
    function GetTraceLevel: integer;
    procedure SetTraceLevel(const Value: integer);
    function GetFileName: string;
    function GetTraceName: string;
    function GetTraceType: boolean;
    procedure SetFileName(const Value: string);
    procedure SetTraceName(const Value: string);
    procedure SetTraceType(const Value: boolean);
  protected
    procedure DoCreate; override;
  public
    // Renvoie la liste des providers qui ont t selectionns pour la trace.
    procedure GetSelectedProviders(lst : TStrings);

    // Nom de la trace.
    property TraceName : string read GetTraceName write SetTraceName;

    // Fichier de trace.
    property FileName : string read GetFileName write SetFileName;

    // Indique s'il faut crer une trace temps rel.
    property TraceRealTime : boolean read GetTraceType write SetTraceType;

    // Indique le niveau de trace slectionn.
    property TraceLevel : integer read GetTraceLevel write SetTraceLevel;
  end;

var
  frmCreateTrace: TfrmCreateTrace;

implementation
uses uETWTraceControler, uETWWmiInfo;
{$R *.dfm}
const STATUS_BUSY = 'Occup';
      STATUS_AVAILABLE = 'Disponible';
      STATUS_UNAVAILABLE = 'Non disponible';

      SUBITEM_STATUS = 0;
      SUBITEM_FLAGS = 1;
      SUBITEM_GUID = 2;

procedure TfrmCreateTrace.acBrowseExecute(Sender: TObject);

  // Retourne un nom de fichier par dfaut pour la trace. Ce dernier est
  // compos du nom de la trace et de la date et heure courante. 
  function DefaultFileName : string;
  var dt : TDateTime;
      year, month,day : word;
      h,min,s, ms : word;
  begin
    dt := Now;
    DecodeDate(dt, year, month, day);
    DecodeTime(dt, h, min, s, ms);
    result := Format('%s_%0.4d-%0.2d-%0.2d_%0.2d-%0.2d.etl', [eTraceName.Text, year, month,day, h, min]);
  end;
  
begin
  if eFileName.Text<>''
  then opDlg.FileName := eFileName.Text
  else opDlg.FileName := DefaultFileName;

  if opDlg.Execute(Handle)
  then eFileName.Text := opDlg.FileName;
end;

procedure TfrmCreateTrace.acBrowseUpdate(Sender: TObject);
begin
  // On ne peut choisir le fichier de trace que si on a choisit une trace
  // fichier.
  TAction(Sender).Enabled := rdbtFile.Checked;
  lblTraceFile.Enabled := rdbtFile.Checked;
  eFileName.Enabled := rdbtFile.Checked;
end;

procedure TfrmCreateTrace.acCancelExecute(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

procedure TfrmCreateTrace.acCreateExecute(Sender: TObject);
begin
  ModalResult := mrOk;
end;

procedure TfrmCreateTrace.acCreateUpdate(Sender: TObject);
begin
  // On ne peut valider la cration de la trace que si les paramtres sont
  // valides :
  // - Nom de trace dfini.
  // - Trace temps rel ou Nom de fichier de trace dfini.
  TAction(Sender).Enabled := (Trim(eTraceName.Text)<>'') and
                             (rdbtRT.Checked or
                              (Trim(eFileName.Text)<>''));
end;

procedure TfrmCreateTrace.Dfinirlesflagsdactivation1Click(Sender: TObject);
var s : string;
    v : cardinal;
    i : integer;
begin
  if lvProviders.ItemFocused<>nil
  then begin
    s := lvProviders.ItemFocused.SubItems[SUBITEM_FLAGS];
    if InputQuery('Flags d''activation', 'Valeur des flags :', s)
    then begin
      val(s, v, i);
      lvProviders.ItemFocused.SubItems[SUBITEM_FLAGS] := Format('$%0.8x', [v]);
    end;
  end;
end;

procedure TfrmCreateTrace.DoCreate;
var list : TStrings;
    info : TETWWMIInfo;
    i : integer;
    j : integer;
    item : TListItem;
    name : string;
    exists : boolean;
    providerGUID : string;
    Enabled : boolean;
begin
  inherited DoCreate;
  // Lors de la cration de la fiche, il faut initialiser la liste des providers
  // ETW disponibles.
  lvProviders.Items.BeginUpdate;
  try
    lvProviders.Items.Clear;
    info := WmiInfo;
    list := TStringList.Create;

    // On commence par gnrer la liste de tous les providers dclars dans ETW
    // Cette liste ne contient que les providers dj en cours d'excution.
    TETWTraceControler.EnumerateRegisteredProviders(List);
    for i := 0 to list.Count -1 do
    begin
      TETWTraceControler.DecodeProviderInfo(list[i], providerGUID, Enabled);

      // Pour chaque provider trouv, on recherche si un nom publique a t
      // dfinit dans Wmi, avec Info.GetProviderName. Si on n'a pas de nom
      // ce sera le GUID qui sera affich.
      item := lvProviders.Items.Add;
      item.Caption := info.GetProviderName(providerGUID);

      // Enfin on indique si le provider est dj en cours d'utilisation.
      if Enabled
      then item.SubItems.Add(STATUS_BUSY)
      else item.SubItems.Add(STATUS_AVAILABLE);
      item.SubItems.Add('$00000000');
      item.SubItems.Add(providerGUID);
    end;

    // On complte la liste avec celle des providers qui ont publis la structure
    // de leurs vnements dans Wmi. En effet ces derniers peuvent tre lancs
    // aprs que la trace ne soit dmarre.
    for i := 0 to info.Providers.Count -1 do
    begin
      name := TWmiObject(info.Providers[i]).Description;
      exists := false;
      // On regarde si le provider n'a pas dj t ajout  la liste.
      for j := 0 to lvProviders.Items.Count-1 do
      begin
        if lvProviders.Items[j].Caption = name
        then begin
          exists := true;
          break;
        end;
      end;

      // Si ce n'est pas le cas, on l'ajoute.
      if not exists
      then begin
        item := lvProviders.Items.Add;
        item.Caption := name;
        item.SubItems.Add(STATUS_UNAVAILABLE);
        item.SubItems.Add('$00000000');
        item.SubItems.Add(TWmiObject(info.Providers[i]).Guid);
      end;
    end;
  finally
    lvProviders.Items.EndUpdate;
  end;
end;

function TfrmCreateTrace.GetFileName: string;
begin
  // Renvoie le nom du fichier de trace. Pour les traces temps rel, on renvoie
  // une chaine vide.
  if TraceRealTime
  then result := ''
  else result := eFileName.Text;
end;

// Renvoie la liste des providers qui ont t selectionns pour la trace.
procedure TfrmCreateTrace.GetSelectedProviders(lst: TStrings);
var i : integer;
begin
  Assert(Assigned(lst));
  lst.Clear;
  for i := 0 to lvProviders.Items.Count -1 do
  begin
    if lvProviders.Items[i].Checked
    then lst.Add(lvProviders.Items[i].SubItems[SUBITEM_GUID] + #9 + lvProviders.Items[i].SubItems[SUBITEM_FLAGS]);
  end; 
end;

function TfrmCreateTrace.GetTraceLevel: integer;
begin
  result := cbTraceLevel.ItemIndex;
end;

function TfrmCreateTrace.GetTraceName: string;
begin
  result := eTraceName.Text;
end;

function TfrmCreateTrace.GetTraceType: boolean;
begin
  result := rdbtRT.Checked;
end;

// Cette fonction permet de trier les providers dans la listView.
procedure TfrmCreateTrace.lvProvidersCompare(Sender: TObject; Item1,
  Item2: TListItem; Data: Integer; var Compare: Integer);
  function CompareStatus : integer;
  var i1, i2 : integer;
  begin
    if item1.SubItems[SUBITEM_STATUS] = STATUS_AVAILABLE
    then i1 := 0
    else begin
      if item1.SubItems[SUBITEM_STATUS] = STATUS_UNAVAILABLE
      then i1 := 1
      else i1 := 2;
    end;

    if item2.SubItems[SUBITEM_STATUS] = STATUS_AVAILABLE
    then i2 := 0
    else begin
      if item2.SubItems[SUBITEM_STATUS] = STATUS_UNAVAILABLE
      then i2 := 1
      else i2 := 2;
    end;
    result := i1-i2;
  end;

  function CompareGuid : integer;
  var s1, s2 : string;
  begin
    s1 := Item1.Caption;
    s2 := Item2.Caption;
    if s1 = ''
    then result := -1
    else begin
      if s2 = ''
      then result := 1
      else begin
        if s1[1] = '{'
        then result := 1
        else begin
          if s2[1] = '{'
          then result := -1
          else result := CompareText(s1, s2);
        end;
      end;
    end;
  end;

  function CompareGuidType : integer;
  var s1, s2 : string;
  begin
    s1 := Item1.Caption;
    s2 := Item2.Caption;
    if s1 = ''
    then result := -1
    else begin
      if s2 = ''
      then result := 1
      else begin
        if s1[1] = '{'
        then result := 1
        else begin
          if s2[1] = '{'
          then result := -1
          else result := 0;
        end;
      end;
    end;
  end;

begin
  Compare := CompareGuidType;
  if Compare = 0
  then Compare := CompareStatus;
  if Compare = 0
  then CompareGuid;
end;

procedure TfrmCreateTrace.SetFileName(const Value: string);
begin
  eFileName.Text := value;
end;

procedure TfrmCreateTrace.SetTraceLevel(const Value: integer);
begin
  cbTraceLevel.ItemIndex := value;
end;

procedure TfrmCreateTrace.SetTraceName(const Value: string);
begin
  eTraceName.Text := value;
end;

procedure TfrmCreateTrace.SetTraceType(const Value: boolean);
begin
  rdbtRT.Checked := value;
end;

end.
