{******************************************************************************}
{                                                                              }
{ ETWTraceViewer                                                               }
{                                                                              }
{ Copyright (C) 2008 Franck SORIANO                                            }
{                                                                              }
{ This program is free software; you can redistribute it and/or                }
{  modify it under the terms of the GNU General Public License                 }
{ as published by the Free Software Foundation; either version 2               }
{ of the License, or (at your option) any later version.                       }
{                                                                              }
{ This program is distributed in the hope that it will be useful,              }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of               }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                }
{ GNU General Public License for more details.                                 }
{                                                                              }
{ You should have received a copy of the GNU General Public License            }
{ along with this program; if not, write to the Free Software                  }
{ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,MA  02110-1301,USA.}
{                                                                              }
{******************************************************************************}
unit uMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, ActnList, XPMan, ComCtrls, ToolWin, ImgList,
  uETWTraceControler;

type
  TfrmMain = class(TForm)
    mnuMain: TMainMenu;
    Fichier1: TMenuItem;
    Crerunenouvelletrace1: TMenuItem;
    N1: TMenuItem;
    Quiter1: TMenuItem;
    Fournisseurs1: TMenuItem;
    AfficherlalistedesTracesactives1: TMenuItem;
    Afficherlaliste1: TMenuItem;
    Arrterlatrace1: TMenuItem;
    ActionList1: TActionList;
    acQuit: TAction;
    acNewTrace: TAction;
    acStopTrace: TAction;
    XPManifest1: TXPManifest;
    acOpenTraceFile: TAction;
    opDlg: TOpenDialog;
    N2: TMenuItem;
    Ouvrirunfichierdetrace1: TMenuItem;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    imgList: TImageList;
    acClear: TAction;
    ToolButton6: TToolButton;
    acQuickStart: TAction;
    ToolButton7: TToolButton;
    ToolButton8: TToolButton;
    acSaveTrace: TAction;
    ToolButton9: TToolButton;
    acSort: TAction;
    N3: TMenuItem;
    Apropos1: TMenuItem;
    procedure Apropos1Click(Sender: TObject);
    procedure acSortExecute(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure acSaveTraceExecute(Sender: TObject);
    procedure acSaveTraceUpdate(Sender: TObject);
    procedure acQuickStartUpdate(Sender: TObject);
    procedure acQuickStartExecute(Sender: TObject);
    procedure acClearExecute(Sender: TObject);
    procedure acClearUpdate(Sender: TObject);
    procedure acOpenTraceFileExecute(Sender: TObject);
    procedure acStopTraceExecute(Sender: TObject);
    procedure acNewTraceExecute(Sender: TObject);
    procedure acStopTraceUpdate(Sender: TObject);
    procedure acQuitExecute(Sender: TObject);
    procedure AfficherlalistedesTracesactives1Click(Sender: TObject);
    procedure Afficherlaliste1Click(Sender: TObject);
  private
    // Contrleur de la trace active.
    FCurrentTrace : TETWTraceControler;

  protected
    procedure DoCreate; override;
    procedure DoDestroy; override;
  public
  end;

var
  frmMain: TfrmMain;

implementation
uses
  uProviderList, uTraceList, uCreateTrace, uETWClient, uTraceView,
  uETWWmiInfo, uETWAdvClient;
{$R *.dfm}
const
  DefaultLogger : TGUID = '{708A0B60-6786-460B-96A1-2C569C7B0B4C}';
  SQLLoggerGUID : TGUID = '{E356181E-4059-4BBA-BA4A-87280762A798}';
  NetGenericGUID : TGUID = '{2F5162CC-79F0-4181-B42F-B98B6CF8FABC}';
  
const
  QuickTraceName = 'Quick Trace';
            
procedure TfrmMain.acClearExecute(Sender: TObject);
begin
  // Demande  la fentre active d'effacer sa trace.
  TfrmTraceView(ActiveMDIChild).ClearTrace;
end;

procedure TfrmMain.acClearUpdate(Sender: TObject);
begin
  // On ne peut effacer une trace que s'il existe une fentre MDI active, de
  // type TfrmTraceView et que cette dernire affiche une trace temps rel.  
  TAction(Sender).Enabled := Assigned(ActiveMDIChild) and
                             (ActiveMDIChild is TfrmTraceView) and
                              (TfrmTraceView(ActiveMDIChild).Controler<>nil);
end;

procedure TfrmMain.acNewTraceExecute(Sender: TObject);
var dlg : TfrmCreateTrace;
    lst : TStrings;
    i : integer;
    frmView : TfrmTraceView;
    Client : TETWClient;
    n : integer;
    ProviderGUID : string;
    ProviderFlags : string;
    Flags : cardinal;
    
begin
  // Pour crer une nouvelle trace, on commence par demander les paramtres de
  // la trace  l'utilisateur.
  dlg := TfrmCreateTrace.Create(self);
  try
    if dlg.ShowModal = mrOk
    then begin
      // On va dmarrer une nouvelle trace. On commence par arrter toute
      // trace en cours d'excution.
      if Assigned(FCurrentTrace)
      then FCurrentTrace.Free;

      // Puis on cre la nouvelle trace.
      FCurrentTrace := TETWTraceControler.Create(dlg.TraceName,
                                                 dlg.FileName);

      // A prsent, il faut activer les providers slectionns.                                                 
      lst := TStringList.Create;
      try
        dlg.GetSelectedProviders(lst);
        for i := 0 to lst.Count-1 do
        begin
          n := pos(#9, lst[i]);
          ProviderGUID := copy(lst[i], 1, n-1);
          ProviderFlags := copy(lst[i], n+1, length(lst[i]));
          val(ProviderFlags, Flags, n);
          FCurrentTrace.EnableProvider(StringToGuid(ProviderGUID),
            dlg.TraceLevel, Flags);
        end;
      finally
        lst.free;
      end;
      
      if dlg.FileName = '' // Il s'agit d'une trace temps rel. Il faut afficher
      then begin // le client.
        // On ouvre la trace temps rel.
        Client := TETWAdvClient.Create(dlg.TraceName, true);

        // On cre une nouvelle fentre TFrlTraceView pour afficher la trace.
        Application.CreateForm(TFrmTraceView, frmView);
        frmView.Client := Client; // On indique la trace  afficher.
        frmView.Controler := @FCurrentTrace; // On indique le pointeur sur le controleur.
        frmView.Caption := 'Trace temps rel : ' + dlg.TraceName;
        frmView.Show;
      end;
    end;
  finally
    dlg.Free;
  end;
end;

procedure TfrmMain.acOpenTraceFileExecute(Sender: TObject);
var Client : TETWClient;
    frmView : TfrmTraceView;
begin
  if opDlg.Execute(Handle)
  then begin
    // Pour ouvrir une trace fichier, il suffit de se connecter sur la trace
    //  partir du fichier.
    Client := TETWAdvClient.Create(opDlg.FileName, false);

    // Et de crer une nouvelle fentre TFrmTraceView pour afficher la trace.
    Application.CreateForm(TfrmTraceView, frmView);
    frmView.Client := Client;
    frmView.Caption := opDlg.FileName;
    frmView.Show;
  end;
end;

procedure TfrmMain.acQuickStartExecute(Sender: TObject);
var frmView : TFrmTraceView;
    Client : TETWClient;
begin
  // Le dmarrage rapide consiste  dmarrer la mme trace que celle de
  // ETWSQLProfiler.

  // Arrte toute trace en cours.
  if Assigned(FCurrentTrace)
  then FCurrentTrace.Free;

  // Cre une nouvelle trace.
  FCurrentTrace := TETWTraceControler.Create(QuickTraceName, '');

  // Active les provider DefaultLogger et SQLLoggerGUID
  FCurrentTrace.EnableProvider(DefaultLogger, 0, 0);
  FCurrentTrace.EnableProvider(SQLLoggerGUID, 0, 0);
  FCurrentTrace.EnableProvider(NetGenericGUID, 0, 0);  

  // Ouvre la trace temps rel.  
  Client := TETWAdvClient.Create(QuickTraceName, true);

  // Il ne reste plus qu' crer une nouvelle fentre TFrmTraceView pour afficher
  // la trace.
  Application.CreateForm(TFrmTraceView, frmView);
  frmView.Client := Client;
  frmView.Controler := @FCurrentTrace;
  frmView.Caption := 'Trace temps rel : ' + QuickTraceName;
  frmView.Show;
end;

procedure TfrmMain.acQuickStartUpdate(Sender: TObject);
begin
  // La trace rapide ne peut tre dmarre que si aucune trace ne s'excute
  // dj.
  TAction(Sender).Enabled := not Assigned(FCurrentTrace);
end;

procedure TfrmMain.acQuitExecute(Sender: TObject);
begin
  Application.Terminate;
end;

procedure TfrmMain.acSaveTraceExecute(Sender: TObject);
begin
  // Exporte la trace dans un fichier text.
  TfrmTraceView(ActiveMDIChild).SaveToFile;
end;

procedure TfrmMain.acSaveTraceUpdate(Sender: TObject);
begin
  TAction(Sender).Enabled := (ActiveMDIChild<>nil) and
                             (ActiveMDIChild is TfrmTraceView);
end;

procedure TfrmMain.acSortExecute(Sender: TObject);
begin
  TfrmTraceView(ActiveMDIChild).SortTrace;
end;

procedure TfrmMain.acStopTraceExecute(Sender: TObject);
begin
  // Pour arrter la trace, il suffit de dtruire son contrleur.
  FCurrentTrace.Free;
  FCurrentTrace := nil;
  if (ActiveMDIChild<>nil) and
     (ActiveMDIChild is TfrmTraceView)
  then TfrmTraceView(ActiveMDIChild).StopTrace;
end;

procedure TfrmMain.acStopTraceUpdate(Sender: TObject);
begin
  TAction(Sender).Enabled := Assigned(FCurrentTrace);
end;

procedure TfrmMain.Afficherlaliste1Click(Sender: TObject);
begin
  if Assigned(frmProviderList)
  then frmProviderList.Free; // On dtruit la fiche pour la recrer et la
                             // rafraichir.

  Application.CreateForm(TfrmProviderList, frmProviderList);
  frmProviderList.Show;
end;

procedure TfrmMain.AfficherlalistedesTracesactives1Click(Sender: TObject);
begin
  if Assigned(frmTraceList)
  then frmTraceList.Free; // On dtruit la fiche pour la recrer et la
                          // rafraichir.

  Application.CreateForm(tfrmTraceList, frmTraceList);
  frmTraceList.Show;
end;

procedure TfrmMain.Apropos1Click(Sender: TObject);
begin
  MessageDlg('ETWTraceViewer'#13#10+
             '(c) 2008, Franck SORIANO',
    mtInformation, [mbOk], 0);
end;

procedure TfrmMain.DoCreate;
begin
  inherited DoCreate;
end;

procedure TfrmMain.DoDestroy;
begin
  if Assigned(FCurrentTrace)
  then FCurrentTrace.Free;
  
  inherited DoDestroy;
end;

procedure TfrmMain.FormActivate(Sender: TObject);
begin
  // On s'assure que la fentre a bien finit de se redessine.
  Application.ProcessMessages;
  WmiInfo; // On charge les donnes Wmi.
  OnActivate := nil; // On dsactive l'vnement pour ne pas refaire le traitement.
end;

end.
