{******************************************************************************}
{                                                                              }
{ ETWTraceViewer                                                               }
{                                                                              }
{ Copyright (C) 2008 Franck SORIANO                                            }
{                                                                              }
{ This program is free software; you can redistribute it and/or                }
{  modify it under the terms of the GNU General Public License                 }
{ as published by the Free Software Foundation; either version 2               }
{ of the License, or (at your option) any later version.                       }
{                                                                              }
{ This program is distributed in the hope that it will be useful,              }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of               }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                }
{ GNU General Public License for more details.                                 }
{                                                                              }
{ You should have received a copy of the GNU General Public License            }
{ along with this program; if not, write to the Free Software                  }
{ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,MA  02110-1301,USA.}
{                                                                              }
{******************************************************************************}
unit uTraceView;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Grids, ExtCtrls, StdCtrls, ComCtrls,
  uETWClient, uETW, uETWTraceControler;

type
  TfrmTraceView = class(TForm)
    grd: TDrawGrid;
    Splitter1: TSplitter;
    Panel1: TPanel;
    Label1: TLabel;
    eEventClassGUID: TEdit;
    lblEventClassName: TLabel;
    Label2: TLabel;
    lblTimeFormat: TLabel;
    Label3: TLabel;
    mUserData: TMemo;
    tmRefresh: TTimer;
    svDlg: TSaveDialog;
    lblCourier: TLabel;
    sb: TStatusBar;
    procedure grdClick(Sender: TObject);
    procedure tmRefreshTimer(Sender: TObject);
    procedure grdSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure FormDestroy(Sender: TObject);
    procedure grdDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect;
      State: TGridDrawState);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    FClient : TETWClient; // La trace  afficher.
    FControler : pointer; // Pointeur sur le contrleur de la fentre principale.
    procedure SetControler(const Value: pointer);
    procedure SetClient(const Value: TETWClient);

    procedure SetupGrid;

    // Mthode utilitaire pour dessiner du texte dans la grille.
    procedure DrawText(Rect: TRect; State: TGridDrawState; Text : string; TextFormat : TTextFormat);

    // Mthode utilitaire pour dessiner un timestamp dans la grille.
    procedure DrawTimeStamp(Rect: TRect; State: TGridDrawState; ts : int64);

    // Mthode utilitaire pour dessiner un entier dans la grille.
    procedure DrawInteger(Rect: TRect; State: TGridDrawState; ts : cardinal);

    // Mthode utilitaire pour dessiner le temps coul depuis la ligne prcdente.
    procedure DrawDelta(Rect: TRect; State: TGridDrawState; ts : cardinal);

    // Affiche l'vnement slectionn dans la partie infrieure de la fentre.
    // <nr> doit dsigner le numro de l'vnement  afficher.
    procedure DisplayEvent(nr : integer);

    // Rafraichit l'affichage de la barre d'tat.
    procedure UpdateStatusBar;
  public
    // Efface les donnes de la trace.
    procedure ClearTrace;

    // Exporte la trace dans un fichier text.
    procedure SaveToFile;

    // Trie la trace par ordre chronologique.
    procedure SortTrace;

    // Ragit  l'arret de la trace.
    procedure StopTrace;

    // La trace a afficher.
    property Client : TETWClient read FClient write SetClient;

    // Pointeur sur la variable qui rfrence le contrleur de la trace.
    // Controler n'est dfinit que pour les trace temps rel.
    property Controler : pointer read FControler write SetControler;
  end;

var
  frmTraceView: TfrmTraceView;

implementation
uses uETWWmiInfo, uETWAdvClient, uTraceData, uStopTrace;
{$R *.dfm}

const
  COL_LIGNE = 0;
  COL_CLOCKTIME = 1;
  COL_DELTA = 2;
  COL_EVENTCLASS = 3;
  COL_EVENTTYPE = 4;
  COL_USERDATA = 5;

// Efface les donnes de la trace.      
procedure TfrmTraceView.ClearTrace;
begin
  if Assigned(FClient)
  then FClient.ClearTrace;
end;

// Affiche l'vnement slectionn dans la partie infrieure de la fentre.
// <nr> doit dsigner le numro de l'vnement  afficher.
procedure TfrmTraceView.DisplayEvent(nr: integer);
var Event : TTraceData;
    Info : TWmiObject;
    i : integer;
    offset : cardinal;
    descr : string;
    prop : TMOFField;
    sHex : string;
    sAscii : string;
begin
  // Premirement, on vrifie que l'vnement existe.
  if Assigned(FClient) and (nr<FClient.Trace.Count)
  then begin
    // On rcupre l'vnement  afficher.
    Event := FClient.Trace[nr];

    // On rcupre la classe qui dcrit la structure de l'vnement.
    Info := Event.GetInfo;


    descr := '';
    Offset := 0;

    // On affiche les informations gnriques de chaque vnement.
    eEventClassGUID.Text := GuidToString(Event.EventClass);
    lblEventClassName.Caption := Event.GetEventClass;
    lblTimeFormat.Caption := DateTimeToStr(FClient.GetDateTime(Event.TimeStamp));

    if Assigned(Info) // Si on a trouv la structure de l'vnement, on peut la
    then begin        // dcoder.
      mUserData.Lines.BeginUpdate;
      try
        mUserData.Lines.Clear;

        // On affiche chaque champ de l'vnement sous la forme :
        // - Nom du champ : Valeur dcode.
        for i := 0 to Info.Properties.Count -1 do
        begin
          prop := TMofFIeld(Info.Properties.Objects[i]);
          mUserData.Lines.Add(prop.DisplayLabel + ' : ' + prop.GetValue(@Event.UserData[0], high(Event.UserData)+1, Offset));
        end;
      finally
        mUserData.Lines.EndUpdate;
      end;
    end
    else begin
      // On ne connait pas la structure de l'vnement. Dans ce cas, on va faire
      // un dump hexadcimal des donnes.
      // Le dump est cependant prsent sous la forme : code hexa, valeur ascii.
      // Le tout sous la forme d'un tableau de 16 octets de large.
      mUserData.Lines.BeginUpdate;
      try
        mUserData.Lines.Clear;
        sHex := '';
        sAscii := '';
        for i := 0 to high(Event.UserData) do
        begin
          // Calcul de l'affichage hexadcimal.
          sHex := sHex + Format('%0.2x ', [Event.UserData[i]]);

          // Calcul de l'affichage ascii.
          // On remplace les caractres non affichables par un espace.
          if Event.UserData[i]>=32
          then sAscii := sAscii + chr(Event.UserData[i])
          else sAscii := sAscii + ' ';

          // On revient  la ligne tous les 16 octets.
          if i mod 16 = 15
          then begin
            mUserData.Lines.Add(sHex + sAscii);
            sHex := '';
            sAscii := '';
          end;
        end;

        // On est arriv  la fin du tableau, il faut afficher la dernire ligne
        // qui est probablement incomplte.
        if sHex<>''
        then begin
          sHex := sHex + StringOfChar(' ', 16*3-length(sHex));
          mUserData.Lines.Add(sHex + sAscii);
        end;
      finally
        mUserData.Lines.EndUpdate;
      end;
    end;
  end
  else begin
    // L'vnement n'existe pas, on n'affiche aucune information.
    eEventClassGUID.Text := '';
    lblTimeFormat.Caption := '';
    lblEventClassName.Caption := '';
    mUserData.Lines.Clear;
  end;
end;

// Mthode utilitaire pour dessiner le temps coul depuis la ligne prcdente.
procedure TfrmTraceView.DrawDelta(Rect: TRect; State: TGridDrawState;
  ts: cardinal);
var s : string;
begin
  Rect.Left := Rect.Left + 1;
  Rect.Top := Rect.Top + 1;
  Rect.Right := rect.Right -1;
  s := Format('%.4f', [ts / 10000]);
  grd.Canvas.TextRect(Rect, s, [tfRight]);
end;

// Mthode utilitaire pour dessiner un entier dans la grille.
procedure TfrmTraceView.DrawInteger(Rect: TRect; State: TGridDrawState;
  ts: cardinal);
var s : string;
begin
  Rect.Left := Rect.Left + 1;
  Rect.Top := Rect.Top + 1;
  Rect.Right := rect.Right -1;
  s := IntToStr(ts);
  grd.Canvas.TextRect(Rect, s, [tfRight]);
end;

    // Mthode utilitaire pour dessiner du texte dans la grille.
procedure TfrmTraceView.DrawText(Rect: TRect; State: TGridDrawState;
  Text: string; TextFormat : TTextFormat);
begin
  Rect.Left := Rect.Left + 1;
  Rect.Top := Rect.Top + 1;
  Rect.Right := rect.Right -1;
  grd.Canvas.TextRect(Rect, Text, TextFormat);
end;

// Mthode utilitaire pour dessiner un timestamp dans la grille.
procedure TfrmTraceView.DrawTimeStamp(Rect: TRect; State: TGridDrawState;
  ts: int64);
var s : string;
begin
  s := DateTimeToStr(FClient.GetDateTime(ts));
  Rect.Left := Rect.Left + 1;
  Rect.Top := Rect.Top + 1;
  Rect.Right := rect.Right -1;
  grd.Canvas.TextRect(Rect, s, [tfRight]);
end;

procedure TfrmTraceView.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
  if Assigned(FControler) // Il faut arrter la trace temps rel
  then begin
    TETWTraceControler(FControler^).Free;
    TETWTraceControler(FControler^) := nil;
    FControler := nil;
  end;
end;

procedure TfrmTraceView.FormDestroy(Sender: TObject);
var dlg : TfrmStopTrace;
begin
  // On dtruit la trace.
  if Assigned(FClient)
  then begin
    dlg := TfrmStopTrace.Create(self);
    try
      dlg.Show;
      Application.ProcessMessages;
      FClient.Free;
    finally
      dlg.Free;
    end;
  end;
end;

procedure TfrmTraceView.grdClick(Sender: TObject);
begin
  // La slection a chang dans la grille, il faut mettre  jour la barre
  // d'tat.
  UpdateStatusBar;
end;

procedure TfrmTraceView.grdDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
var TraceData : TTraceData;  
begin
  if ARow = 0
  then begin
    case ACol of
    COL_LIGNE: DrawText(Rect, State, 'Ligne', [tfCenter]);
    COL_CLOCKTIME: DrawText(Rect, State, 'ClockTime', [tfCenter]);
    COL_DELTA: DrawText(Rect, State, 'Delta', [tfCenter]);
    COL_EVENTCLASS: DrawText(Rect, State, 'EventClass', [tfCenter]);
    COL_EVENTTYPE: DrawText(Rect, State, 'EventType', [tfCenter]);
    COL_USERDATA: DrawText(Rect, State, 'User Data', [tfCenter]);
    end;
  end
  else begin
    if Not Assigned(FClient) or (ARow-1>=FClient.Trace.Count)
    then DrawText(Rect, State, '', [])
    else begin
      TraceData := TTraceData(FClient.Trace[ARow-1]);
      TraceData.Setup;
      if ACol<>5
      then grd.Canvas.Font := grd.Font
      else grd.Canvas.Font := lblCourier.Font;
      if gdSelected in State
      then grd.Canvas.Font.Color := clWhite
      else grd.Canvas.Font.Color := clBlack;
      
      case ACol of
      COL_LIGNE:
        DrawInteger(Rect, State, ARow);
      COL_CLOCKTIME:
        DrawTimeStamp(Rect, State, TraceData.TimeStamp);
      COL_DELTA:
        if ARow-1>0
        then DrawDelta(Rect, State, (TraceData.TimeStamp-TTraceData(FClient.Trace[ARow-2]).TimeStamp))
        else DrawDelta(Rect, State, TraceData.TimeStamp-TTraceData(FClient.Trace[0]).TimeStamp);
      COL_EVENTCLASS:
        DrawText(Rect, State, TraceData.GetEventClass, [tfCenter]);
      COL_EVENTTYPE:
        DrawText(Rect, State, TraceData.GetEventType, [tfCenter]);
      COL_USERDATA:
        // On affiche les donnes utilisateur, indentes en fonction du thread
        // qui a gnr le message. De cette faon en lecture de la trace on
        // peut immdiatement rprer les sequences d'vnements gnres par un
        // mme thread.
        DrawText(Rect, State, StringOfChar(' ', TraceData.ThreadCol*3) + TraceData.GetUserData, [])
      end;
    end;
  end;
end;

procedure TfrmTraceView.grdSelectCell(Sender: TObject; ACol, ARow: Integer;
  var CanSelect: Boolean);
begin
  // On a slectionner un vnement, il faut l'afficher dans la zone infrieure.
  CanSelect := true;
  DisplayEvent(ARow-1);
end;

procedure TfrmTraceView.SaveToFile;
begin
  // Exporte la trace dans un fichier texte. Le fichier pourra ensuite tre
  // facilement import dans Excel.
  if Assigned(FClient) and (svDlg.Execute(Handle))
  then TETWAdvClient(FClient).SaveToFile(svDlg.FileName);
end;

procedure TfrmTraceView.SetClient(const Value: TETWClient);
begin
  FClient := Value;
  SetupGrid;
end;

procedure TfrmTraceView.SetControler(const Value: pointer);
begin
  FControler := Value;
  tmRefresh.Enabled := true;
end;

procedure TfrmTraceView.SetupGrid;
begin
  // Configure la grille d'affichage de la trace.
  grd.ColCount := 6;
  grd.ColWidths[COL_LIGNE] := 50;
  grd.ColWidths[COL_CLOCKTIME] := 110;
  grd.ColWidths[COL_DELTA] := 75;
  grd.ColWidths[COL_EVENTCLASS] := 150;
  grd.ColWidths[COL_EVENTTYPE] := 75;
  grd.ColWidths[COL_USERDATA] := 700;
  grd.DefaultRowHeight := 16;
  if Assigned(FClient)
  then grd.RowCount := FClient.Trace.Count +2;
end;

procedure TfrmTraceView.SortTrace;
begin
  if Assigned(FClient)
  then begin
    FClient.SortTrace;
    grd.Invalidate;
  end;
end;

procedure TfrmTraceView.StopTrace;
var dlg : TfrmStopTrace;
begin
  if Assigned(FClient)
  then begin
    dlg := TfrmStopTrace.Create(self);
    try
      dlg.Show;
      Application.ProcessMessages;
      FClient.StopTrace;
    finally
      dlg.Free;
    end;
  end;
end;

// Rafraichit l'affichage de la barre d'tat.
procedure TfrmTraceView.UpdateStatusBar;
var
  Selfrom, Selto : integer;
  t0, t1 : int64;
begin
  // On rcupre la slection.
  selFrom := grd.Selection.Top;
  SelTo := grd.Selection.Bottom;

  // On affiche les lignes slectinne dans la barre d'tat.
  sb.Panels[0].Text:='Selection : ' + IntToStr(selFrom) + '-' + IntToStr(SelTo);

  if Assigned(FClient)
  then begin
    dec(selFrom);
    dec(selTo);

    // On lit la date du premier vnement slectionn.
    if selFrom<FClient.Trace.Count
    then t0 := TTraceData(FClient.Trace[selFrom]).TimeStamp
    else t0 := 0;

    // On lit la date du dernier vnement slectionn.
    if selTo<FClient.Trace.Count
    then t1 := TTraceData(FClient.Trace[selTo]).TimeStamp
    else t1 := t0;

    // Enfin on affiche la tout
    sb.Panels[1].Text := 'Du : ' + IntToStr(t0); // Date du premier vnement
    sb.Panels[2].Text := 'Au : ' + IntToStr(t1); // Date du dernier vnement

    // Dure coule entre les deux.
    sb.Panels[3].Text := 'Dure : ' + Format('%.4f', [(t1-t0)/10000]) + ' ms';
  end;
end;

procedure TfrmTraceView.tmRefreshTimer(Sender: TObject);
begin
  // Priodiquement, on flush la trace temps rel pour que les vnements soient
  // immdiatement visibles. Sinon il faut attendre la fin de la tempo dfinie
  // dans ETW (1s) ou que les buffers soient pleins.
  if Assigned(FControler) and (Assigned(TETWTraceControler(FControler^)))
  then begin
    TETWTraceControler(FControler^).Flush; // Flush de la trace.
    if grd.RowCount <> FClient.Trace.Count +2   // Mise  jour de la grille.
    then grd.RowCount := FClient.Trace.Count +2;
  end;
end;

end.
