{******************************************************************************}
{                                                                              }
{ Public headers for event tracing control applications,                       }
{    consumers and providersEvent Tracing for Windows SQL Logger Unit          }
{                                                                              }
{ Portions created by Microsoft are Copyright (C) 1995-2001 Microsoft          }
{ Corporation. All Rights Reserved.                                            }
{                                                                              }
{ The contents of this file are subject to the Mozilla Public License          }
{ Version 1.1 (the "License"); you may not use this file except in             }
{ compliance with the License. You may obtain a copy of the License at         }
{ http://www.mozilla.org/MPL/                                                  }
{                                                                              }
{ Software distributed under the License is distributed on an "AS IS"          }
{ basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the      }
{ License for the specific language governing rights and limitations           }
{ under the License.                                                           }
{                                                                              }
{ The Original file is EvnTrace.h. The original pascal code is uSQLLogger.pas  }
{ released February 2008.                                                      }
{                                                                              }
{ The Initial Developer of the pascal code is Franck SORIANO.                  }
{                                                                              }
{ Alternatively, the contents of this file may be used under the terms of the  }
{ GNU Lesser General Public License (the  "LGPL License"), in which case the   }
{ provisions of the LGPL License are applicable instead of those above.        }
{ If you wish to allow use of your version of this file only under the terms   }
{ of the LGPL License and not to allow others to use your version of this file }
{ under the MPL, indicate your decision by deleting  the provisions above and  }
{ replace  them with the notice and other provisions required by the LGPL      }
{ License.  If you do not delete the provisions above, a recipient may use     }
{ your version of this file under either the MPL or the LGPL License.          }
{                                                                              }
{ For more information about the LGPL: http://www.gnu.org/copyleft/lesser.html }
{                                                                              }
{******************************************************************************}

unit uETW;
interface
uses ActiveX, Windows;

//
// EventTraceGuid is used to identify a event tracing session
//
const EventTraceGuid : TGUID = '{68fdd900-4a3e-11d1-84f4-0000f80464e3}';

//
// SystemTraceControlGuid. Used to specify event tracing for kernel
//
const SystemTraceControlGuid : TGUID = '{9e814aad-3204-11d2-9a82-006008a86939}';


//
// EventTraceConfigGuid. Used to report system configuration records
//
const EventTraceConfigGuid : TGUID = '{01853a65-418f-4f36-aefc-dc0f1d2fd235}';

//
// DefaultTraceSecurityGuid. Specifies the default event tracing security
//
const DefaultTraceSecurityGuid : TGUID = '{0811c1af-7a07-4a06-82ed-869455cdf713}';

const
  KERNEL_LOGGER_NAME = 'NT Kernel Logger';
  GLOBAL_LOGGER_NAME = 'GlobalLogger';
  EVENT_LOGGER_NAME = 'EventLog';
  DIAG_LOGGER_NAME = 'DiagLog';

const
  MAX_MOF_FIELDS = 16;  // Limit of USE_MOF_PTR fields

type TRACEHANDLE = int64;
     PTRACEHANDLE = ^TRACEHANDLE;

const
  WMI_GET_ALL_DATA = 0;
  WMI_GET_SINGLE_INSTANCE = 1;
  WMI_SET_SINGLE_INSTANCE = 2;
  WMI_SET_SINGLE_ITEM = 3;
  WMI_ENABLE_EVENTS = 4;
  WMI_DISABLE_EVENTS = 5;
  WMI_ENABLE_COLLECTION = 6;
  WMI_DISABLE_COLLECTION = 7;
  WMI_REGINFO = 8;
  WMI_EXECUTE_METHOD = 9;

  WNODE_FLAG_TRACED_GUID = $00020000;
  WNODE_FLAG_LOG_WNODE = $00040000;
  WNODE_FLAG_USE_GUID_PTR = $00080000;
  WNODE_FLAG_USE_MOF_PTR = $00100000;
  WNODE_FLAG_NO_HEADER = $00200000; 


//types for event data going to System Event Logger
const SYSTEM_EVENT_TYPE                        =1;

//
// predefined generic event types (0x00 to 0x09 reserved).
//
const EVENT_TRACE_TYPE_INFO               =$00;  // Info or point event
const EVENT_TRACE_TYPE_START              =$01;  // Start event
const EVENT_TRACE_TYPE_END                =$02;  // End event
const EVENT_TRACE_TYPE_STOP               =$02;  // Stop event (WinEvent compatible)
const EVENT_TRACE_TYPE_DC_START           =$03;  // Collection start marker
const EVENT_TRACE_TYPE_DC_END             =$04;  // Collection end marker
const EVENT_TRACE_TYPE_EXTENSION          =$05;  // Extension/continuation
const EVENT_TRACE_TYPE_REPLY              =$06;  // Reply event
const EVENT_TRACE_TYPE_DEQUEUE            =$07;  // De-queue event
const EVENT_TRACE_TYPE_RESUME             =$07;  // Resume event (WinEvent compatible)
const EVENT_TRACE_TYPE_CHECKPOINT         =$08;  // Generic checkpoint event
const EVENT_TRACE_TYPE_SUSPEND            =$08;  // Suspend event (WinEvent compatible)
const EVENT_TRACE_TYPE_WINEVT_SEND        =$09;  // Send Event (WinEvent compatible)
const EVENT_TRACE_TYPE_WINEVT_RECEIVE     =$F0;  // Receive Event (WinEvent compatible)

//
// Predefined Event Tracing Levels for Software/Debug Tracing
//
//
// Trace Level is UCHAR and passed in through the EnableLevel parameter
// in EnableTrace API. It is retrieved by the provider using the
// GetTraceEnableLevel macro.It should be interpreted as an integer value
// to mean everything at or below that level will be traced.
//
// Here are the possible Levels.
//

const TRACE_LEVEL_NONE        =0;   // Tracing is not on
const TRACE_LEVEL_CRITICAL    =1;   // Abnormal exit or termination
const TRACE_LEVEL_FATAL       =1;   // Deprecated name for Abnormal exit or termination
const TRACE_LEVEL_ERROR       =2;   // Severe errors that need logging
const TRACE_LEVEL_WARNING     =3;   // Warnings such as allocation failure
const TRACE_LEVEL_INFORMATION =4;   // Includes non-error cases(e.g.,Entry-Exit)
const TRACE_LEVEL_VERBOSE     =5;   // Detailed traces from intermediate steps
const TRACE_LEVEL_RESERVED6   =6;
const TRACE_LEVEL_RESERVED7   =7;
const TRACE_LEVEL_RESERVED8   =8;
const TRACE_LEVEL_RESERVED9   =9;

//
// Event types for Process & Threads
//

const EVENT_TRACE_TYPE_LOAD                  =$0A;      // Load image

//
// Event types for IO subsystem
//

const EVENT_TRACE_TYPE_IO_READ               =$0A;
const EVENT_TRACE_TYPE_IO_WRITE              =$0B;
const EVENT_TRACE_TYPE_IO_READ_INIT          =$0C;
const EVENT_TRACE_TYPE_IO_WRITE_INIT         =$0D;
const EVENT_TRACE_TYPE_IO_FLUSH              =$0E;
const EVENT_TRACE_TYPE_IO_FLUSH_INIT         =$0F;


//
// Event types for Memory subsystem
//

const EVENT_TRACE_TYPE_MM_TF                 =$0A;      // Transition fault
const EVENT_TRACE_TYPE_MM_DZF                =$0B;      // Demand Zero fault
const EVENT_TRACE_TYPE_MM_COW                =$0C;      // Copy on Write
const EVENT_TRACE_TYPE_MM_GPF                =$0D;      // Guard Page fault
const EVENT_TRACE_TYPE_MM_HPF                =$0E;      // Hard page fault
const EVENT_TRACE_TYPE_MM_AV                 =$0F;      // Access violation

//
// Event types for Network subsystem, all protocols
//

const EVENT_TRACE_TYPE_SEND                  =$0A;     // Send
const EVENT_TRACE_TYPE_RECEIVE               =$0B;     // Receive
const EVENT_TRACE_TYPE_CONNECT               =$0C;     // Connect
const EVENT_TRACE_TYPE_DISCONNECT            =$0D;     // Disconnect
const EVENT_TRACE_TYPE_RETRANSMIT            =$0E;     // ReTransmit
const EVENT_TRACE_TYPE_ACCEPT                =$0F;     // Accept
const EVENT_TRACE_TYPE_RECONNECT             =$10;     // ReConnect
const EVENT_TRACE_TYPE_CONNFAIL              =$11;     // Fail
const EVENT_TRACE_TYPE_COPY_TCP              =$12;     // Copy in PendData
const EVENT_TRACE_TYPE_COPY_ARP              =$13;     // NDIS_STATUS_RESOURCES Copy
const EVENT_TRACE_TYPE_ACKFULL               =$14;     // A full data ACK
const EVENT_TRACE_TYPE_ACKPART               =$15;     // A Partial data ACK
const EVENT_TRACE_TYPE_ACKDUP                =$16;     // A Duplicate data ACK


//
// Event Types for the Header (to handle internal event headers)
//

const EVENT_TRACE_TYPE_GUIDMAP                =$0A;
const EVENT_TRACE_TYPE_CONFIG                 =$0B;
const EVENT_TRACE_TYPE_SIDINFO                =$0C;
const EVENT_TRACE_TYPE_SECURITY               =$0D;

//
// Event Types for Registry subsystem
//

const EVENT_TRACE_TYPE_REGCREATE                  =$0A;     // NtCreateKey
const EVENT_TRACE_TYPE_REGOPEN                    =$0B;     // NtOpenKey
const EVENT_TRACE_TYPE_REGDELETE                  =$0C;     // NtDeleteKey
const EVENT_TRACE_TYPE_REGQUERY                   =$0D;     // NtQueryKey
const EVENT_TRACE_TYPE_REGSETVALUE                =$0E;     // NtSetValueKey
const EVENT_TRACE_TYPE_REGDELETEVALUE             =$0F;     // NtDeleteValueKey
const EVENT_TRACE_TYPE_REGQUERYVALUE              =$10;     // NtQueryValueKey
const EVENT_TRACE_TYPE_REGENUMERATEKEY            =$11;     // NtEnumerateKey
const EVENT_TRACE_TYPE_REGENUMERATEVALUEKEY       =$12;     // NtEnumerateValueKey
const EVENT_TRACE_TYPE_REGQUERYMULTIPLEVALUE      =$13;     // NtQueryMultipleValueKey
const EVENT_TRACE_TYPE_REGSETINFORMATION          =$14;     // NtSetInformationKey
const EVENT_TRACE_TYPE_REGFLUSH                   =$15;     // NtFlushKey
const EVENT_TRACE_TYPE_REGKCBCREATE               =$16;     // KcbCreate
const EVENT_TRACE_TYPE_REGKCBDELETE               =$17;     // KcbDelete
const EVENT_TRACE_TYPE_REGKCBRUNDOWNBEGIN         =$18;     // KcbRundownBegin
const EVENT_TRACE_TYPE_REGKCBRUNDOWNEND           =$19;     // KcbRundownEnd
const EVENT_TRACE_TYPE_REGVIRTUALIZE              =$1A;     // VirtualizeKey
const EVENT_TRACE_TYPE_REGCLOSE                   =$1B;     // NtClose (KeyObject)

//
// Event types for system configuration records
//
const EVENT_TRACE_TYPE_CONFIG_CPU             =$0A;     // CPU Configuration
const EVENT_TRACE_TYPE_CONFIG_PHYSICALDISK    =$0B;     // Physical Disk Configuration
const EVENT_TRACE_TYPE_CONFIG_LOGICALDISK     =$0C;     // Logical Disk Configuration
const EVENT_TRACE_TYPE_CONFIG_NIC             =$0D;     // NIC Configuration
const EVENT_TRACE_TYPE_CONFIG_VIDEO           =$0E;     // Video Adapter Configuration
const EVENT_TRACE_TYPE_CONFIG_SERVICES        =$0F;     // Active Services
const EVENT_TRACE_TYPE_CONFIG_POWER           =$10;     // ACPI Configuration
const EVENT_TRACE_TYPE_CONFIG_NETINFO         =$11;     // Networking Configuration

const EVENT_TRACE_TYPE_CONFIG_IRQ             =$15;     // IRQ assigned to devices
const EVENT_TRACE_TYPE_CONFIG_PNP             =$16;     // PnP device info
const EVENT_TRACE_TYPE_CONFIG_IDECHANNEL      =$17;     // Primary/Secondary IDE channel Configuration


//
// Enable flags for Kernel Events
//
const EVENT_TRACE_FLAG_PROCESS            =$00000001;  // process start & end
const EVENT_TRACE_FLAG_THREAD             =$00000002;  // thread start & end
const EVENT_TRACE_FLAG_IMAGE_LOAD         =$00000004;  // image load

const EVENT_TRACE_FLAG_DISK_IO            =$00000100;  // physical disk IO
const EVENT_TRACE_FLAG_DISK_FILE_IO       =$00000200;  // requires disk IO

const EVENT_TRACE_FLAG_MEMORY_PAGE_FAULTS =$00001000;  // all page faults
const EVENT_TRACE_FLAG_MEMORY_HARD_FAULTS =$00002000;  // hard faults only

const EVENT_TRACE_FLAG_NETWORK_TCPIP      =$00010000;  // tcpip send & receive

const EVENT_TRACE_FLAG_REGISTRY           =$00020000;  // registry calls
const EVENT_TRACE_FLAG_DBGPRINT           =$00040000;  // DbgPrint(ex) Calls

//
// Enable flags for Kernel Events on Vista and above
//
const EVENT_TRACE_FLAG_PROCESS_COUNTERS   =$00000008;  // process perf counters
const EVENT_TRACE_FLAG_CSWITCH            =$00000010;  // context switches
const EVENT_TRACE_FLAG_DPC                =$00000020;  // deffered procedure calls
const EVENT_TRACE_FLAG_INTERRUPT          =$00000040;  // interrupts
const EVENT_TRACE_FLAG_SYSTEMCALL         =$00000080;  // system calls

const EVENT_TRACE_FLAG_DISK_IO_INIT       =$00000400;  // physical disk IO initiation

const EVENT_TRACE_FLAG_ALPC               =$00100000;  // ALPC traces
const EVENT_TRACE_FLAG_SPLIT_IO           =$00200000;  // split io traces (VolumeManager)

const EVENT_TRACE_FLAG_DRIVER             =$00800000;  // driver delays
const EVENT_TRACE_FLAG_PROFILE            =$01000000;  // sample based profiling
const EVENT_TRACE_FLAG_FILE_IO            =$02000000;  // file IO
const EVENT_TRACE_FLAG_FILE_IO_INIT       =$04000000;  // file IO initiation

//
// Pre-defined Enable flags for everybody else
//
const EVENT_TRACE_FLAG_EXTENSION          =$80000000;  // Indicates more flags
const EVENT_TRACE_FLAG_FORWARD_WMI        =$40000000;  // Can forward to WMI
const EVENT_TRACE_FLAG_ENABLE_RESERVE     =$20000000;  // Reserved

//
// Logger Mode flags
//
const EVENT_TRACE_FILE_MODE_NONE          =$00000000;  // Logfile is off
const EVENT_TRACE_FILE_MODE_SEQUENTIAL    =$00000001;  // Log sequentially
const EVENT_TRACE_FILE_MODE_CIRCULAR      =$00000002;  // Log in circular manner
const EVENT_TRACE_FILE_MODE_APPEND        =$00000004;  // Append sequential log

const EVENT_TRACE_REAL_TIME_MODE          =$00000100;  // Real time mode on
const EVENT_TRACE_DELAY_OPEN_FILE_MODE    =$00000200;  // Delay opening file
const EVENT_TRACE_BUFFERING_MODE          =$00000400;  // Buffering mode only
const EVENT_TRACE_PRIVATE_LOGGER_MODE     =$00000800;  // Process Private Logger
const EVENT_TRACE_ADD_HEADER_MODE         =$00001000;  // Add a logfile header

const EVENT_TRACE_USE_GLOBAL_SEQUENCE     =$00004000;  // Use global sequence no.
const EVENT_TRACE_USE_LOCAL_SEQUENCE      =$00008000;  // Use local sequence no.

const EVENT_TRACE_RELOG_MODE              =$00010000;  // Relogger

const EVENT_TRACE_USE_PAGED_MEMORY        =$01000000;  // Use pageable buffers

//
// Logger Mode flags on XP and above
//
const EVENT_TRACE_FILE_MODE_NEWFILE       =$00000008;  // Auto-switch log file
const EVENT_TRACE_FILE_MODE_PREALLOCATE   =$00000020;  // Pre-allocate mode

//
// Logger Mode flags on Vista and above
//
const EVENT_TRACE_NONSTOPPABLE_MODE       =$00000040;  // Session cannot be stopped (Autologger only)
const EVENT_TRACE_SECURE_MODE             =$00000080;  // Secure session
const EVENT_TRACE_USE_KBYTES_FOR_SIZE     =$00002000;  // Use KBytes as file size unit
const EVENT_TRACE_PRIVATE_IN_PROC         =$00020000;  // In process private logger
const EVENT_TRACE_MODE_RESERVED           =$00100000;  // Reserved bit, used to signal Heap/Critsec tracing


//
// ControlTrace Codes
//
const EVENT_TRACE_CONTROL_QUERY           =0;
const EVENT_TRACE_CONTROL_STOP            =1;
const EVENT_TRACE_CONTROL_UPDATE          =2;

//
// Flush ControlTrace Codes for XP and above
//
const EVENT_TRACE_CONTROL_FLUSH           =3;       // Flushes all the buffers

//
// Flags used by WMI Trace Message
// Note that the order or value of these flags should NOT be changed as they are processed
// in this order.
//
const TRACE_MESSAGE_SEQUENCE              =1;  // Message should include a sequence number
const TRACE_MESSAGE_GUID                  =2;  // Message includes a GUID
const TRACE_MESSAGE_COMPONENTID           =4;  // Message has no GUID, Component ID instead
const TRACE_MESSAGE_TIMESTAMP             =8;  // Message includes a timestamp
const TRACE_MESSAGE_PERFORMANCE_TIMESTAMP =16; // *Obsolete* Clock type is controlled by the logger
const TRACE_MESSAGE_SYSTEMINFO            =32; // Message includes system information TID,PID

//
// Vista flags set by system to indicate provider pointer size.
//

const TRACE_MESSAGE_POINTER32             =$0040;  // Message logged by 32 bit provider
const TRACE_MESSAGE_POINTER64             =$0080;  // Message logged by 64 bit provider

const TRACE_MESSAGE_FLAG_MASK         =$FFFF;  // Only the lower 16 bits of flags are placed in the message
                                                // those above 16 bits are reserved for local processing
const TRACE_MESSAGE_MAXIMUM_SIZE  =8*1024;      // the maximum size allowed for a single trace message
                                                // longer messages will return ERROR_BUFFER_OVERFLOW
//
// Flags to indicate to consumer which fields
// in the EVENT_TRACE_HEADER are valid
//

const EVENT_TRACE_USE_PROCTIME   =$0001;    // ProcessorTime field is valid
const EVENT_TRACE_USE_NOCPUTIME  =$0002;    // No Kernel/User/Processor Times

//
// TRACE_HEADER_FLAG values are used in the Flags field of EVENT_TRACE_HEADER
// structure while calling into TraceEvent API
//

const TRACE_HEADER_FLAG_USE_TIMESTAMP     =$00000200;
const TRACE_HEADER_FLAG_TRACED_GUID       =$00020000; // denotes a trace
const TRACE_HEADER_FLAG_LOG_WNODE         =$00040000; // request to log Wnode
const TRACE_HEADER_FLAG_USE_GUID_PTR      =$00080000; // Guid is actually a pointer
const TRACE_HEADER_FLAG_USE_MOF_PTR       =$00100000; // MOF data are dereferenced


//
// Trace header for all legacy events.
//
type
  handle = Windows.THandle;
  EVENT_TRACE_HEADER = packed record // overlays WNODE_HEADER
    Size : word; // Size of entire record
    HeaderType : byte;  // Header type - internal use only
    MarkerFlags : byte; // Marker - internal use only
    Type_ : byte;
    Level : byte;
    Version : word;
    ThreadId : Cardinal;
    ProcessId : Cardinal;
    TimeStamp : int64;
    Guid : TGUID; // Guid that identifies event
    ClientContext : cardinal;
    Flags : cardinal;
  end;
  PEVENT_TRACE_HEADER=^EVENT_TRACE_HEADER;

//
// This header is used to trace and track transaction co-relations
//
  EVENT_INSTANCE_HEADER = packed record
    Size : word;
    HeaderType : byte;  // Header type - internal use only
    MarkerFlags : byte; // Marker - internal use only
    Version : Cardinal;
    ThreadId : Cardinal;
    ProcessId : Cardinal;
    TimeStamp : int64;
    RegHandle : int64;
    InstanceId : cardinal;
    ParentInstanceId : cardinal;
    ProcessorTime : int64; // Processor Clock
    ParentRegHandle : int64;
  end;
  PEVENT_INSTANCE_HEADER = ^EVENT_INSTANCE_HEADER;


//
// Following are structures and macros for use with USE_MOF_PTR
//
// Trace data types
const ETW_NULL_TYPE_VALUE                 =0;
const ETW_OBJECT_TYPE_VALUE               =1;
const ETW_STRING_TYPE_VALUE               =2;
const ETW_SBYTE_TYPE_VALUE                =3;
const ETW_BYTE_TYPE_VALUE                 =4;
const ETW_INT16_TYPE_VALUE                =5;
const ETW_UINT16_TYPE_VALUE               =6;
const ETW_INT32_TYPE_VALUE                =7;
const ETW_UINT32_TYPE_VALUE               =8;
const ETW_INT64_TYPE_VALUE                =9;
const ETW_UINT64_TYPE_VALUE               =10;
const ETW_CHAR_TYPE_VALUE                 =11;
const ETW_SINGLE_TYPE_VALUE               =12;
const ETW_DOUBLE_TYPE_VALUE               =13;
const ETW_BOOLEAN_TYPE_VALUE              =14;
const ETW_DECIMAL_TYPE_VALUE              =15;
// Extended types
const ETW_GUID_TYPE_VALUE                 =101;
const ETW_ASCIICHAR_TYPE_VALUE            =102;
const ETW_ASCIISTRING_TYPE_VALUE          =103;
const ETW_COUNTED_STRING_TYPE_VALUE       =104;
const ETW_POINTER_TYPE_VALUE              =105;
const ETW_SIZET_TYPE_VALUE                =106;
const ETW_HIDDEN_TYPE_VALUE               =107;
const ETW_BOOL_TYPE_VALUE                 =108;
const ETW_COUNTED_ANSISTRING_TYPE_VALUE   =109;
const ETW_REVERSED_COUNTED_STRING_TYPE_VALUE =110;
const ETW_REVERSED_COUNTED_ANSISTRING_TYPE_VALUE =111;
const ETW_NON_NULL_TERMINATED_STRING_TYPE_VALUE =112;
const ETW_REDUCED_ANSISTRING_TYPE_VALUE   =113;
const ETW_REDUCED_STRING_TYPE_VALUE       =114;
const ETW_SID_TYPE_VALUE                  =115;
const ETW_VARIANT_TYPE_VALUE              =116;
const ETW_PTVECTOR_TYPE_VALUE             =117;
const ETW_WMITIME_TYPE_VALUE              =118;
const ETW_DATETIME_TYPE_VALUE             =119;
const ETW_REFRENCE_TYPE_VALUE             =120;



//
// This is the header for every logfile. The memory for LoggerName
// and LogFileName must be contiguous adjacent to this structure
// Allows both user-mode and kernel-mode to understand the header
//
type
  TRACE_LOGFILE_HEADER = packed record
    BufferSize : Cardinal;
    MajorVersion : byte;
    MinorVersion : byte;
    SubVersion : byte;
    SubMinorVersion : byte;

    ProviderVersion : cardinal;    // defaults to NT version
    NumberOfProcessors : cardinal; // Number of Processors
    EndTime : LARGE_INTEGER;       // Time when logger stops
    TimerResolution : cardinal;
    MaximumFileSize : cardinal;    // Maximum in Mbytes
    LogFileMode : cardinal;        // specify logfile mode
    BuffersWritten : cardinal;     // used to file start of Circular File
    LogInstanceGuid : TGUID;       // For RealTime Buffer Delivery
    LoggerName : PWideChar;
    LogFileName : PWideChar;
    TimeZone : TIME_ZONE_INFORMATION;           // TIME_ZONE_INFORMATION TimeZone;
    BootTime : int64;
    PerfFreq : int64;
    StartTime : int64;
    ReservedFlags : cardinal;
    BuffersLost : cardinal;
  end;
  PTRACE_LOGFILE_HEADER = ^TRACE_LOGFILE_HEADER;

//
// Instance Information to track parent child relationship of Instances.
//
  EVENT_INSTANCE_INFO = packed record
    RegHandle : handle;
    InstanceId : cardinal;
  end;
  PEVENT_INSTANCE_INFO = ^EVENT_INSTANCE_INFO;

  WNODE_HEADER = packed record
    BufferSize : cardinal;
    ProviderId : cardinal;
    Version : cardinal;
    Linkage : cardinal;
    TimeStamp : int64;
    Guid : TGUID;
    ClientContext : cardinal;
    Flags : cardinal;
  end;
  PWNODE_HEADER = ^WNODE_HEADER;
  //
// Structures that have UNICODE and ANSI versions are defined here
//

//
// Logger configuration and running statistics. This structure is used
// by user-mode callers, such as PDH library
//
  EVENT_TRACE_PROPERTIES = packed record
    Wnode : WNODE_HEADER;
//
// data provided by caller
    BufferSize : cardinal;              // buffer size for logging (kbytes)
    MinimumBuffers : cardinal;          // minimum to preallocate
    MaximumBuffers : cardinal;          // maximum buffers allowed
    MaximumFileSize : cardinal;         // maximum logfile size (in MBytes)
    LogFileMode : cardinal;             // sequential, circular
    FlushTimer : cardinal;              // buffer flush timer, in seconds
    EnableFlags :cardinal;              // trace enable flags
    AgeLimit : cardinal;                // age decay time, in minutes

// data returned to caller
    NumberOfBuffers : cardinal;              // no of buffers in use
    FreeBuffers : cardinal;                  // no of buffers free
    EventsLost : cardinal;                   // event records lost
    BuffersWritten : cardinal;               // no of buffers written to file
    LogBuffersLost : cardinal;               // no of logfile write failures
    RealTimeBuffersLost : cardinal;          // no of rt delivery failures
    LoggerThreadId : HANDLE;              // thread id of Logger
    LogFileNameOffset : integer;            // Offset to LogFileName
    LoggerNameOffset : integer;             // Offset to LoggerName
  end;
  PEVENT_TRACE_PROPERTIES = ^EVENT_TRACE_PROPERTIES;

// NOTE:
// If AgeLimit is 0, default is used
// If AgeLimit is < 0, buffer aging is turned off


//
// Data Provider structures
//
// Used by RegisterTraceGuids()
//
  TRACE_GUID_REGISTRATION = packed record
    Guid : pointer;
    RegHandle : Handle;
  end;
  PTRACE_GUID_REGISTRATION = ^TRACE_GUID_REGISTRATION;

//
// Data consumer structures
//
  TRACE_GUID_PROPERTIES = packed record
    Guid : TGUID;
    GuidType : cardinal;
    LoggerId : cardinal;
    EnableLevel : cardinal;
    EnableFlags : cardinal;
    IsEnable : cardinal;
  end;
  PTRACE_GUID_PROPERTIES = ^TRACE_GUID_PROPERTIES;

  ETW_BUFFER_CONTEXT = packed record
    ProcessorNumber : byte;
    Alignment : byte;
    LoggerId : word;
  end;
  PETW_BUFFER_CONTEXT = ^ETW_BUFFER_CONTEXT;

//
// Provider Information Flags used on Vista and above
//
const TRACE_PROVIDER_FLAG_LEGACY = $00000001;
const TRACE_PROVIDER_FLAG_PRE_ENABLE = $00000002;

//
// Enable Information for Provider Instance 
// Used on Vista and above
//
type
  TRACE_ENABLE_INFO = packed record
    IsEnabled : cardinal;
    Level : byte;
    Reserved1 : byte;
    LoggerId : word;
    EnableProperty : cardinal;
    Reserved2 : cardinal;
    MatchAnyKeyword : int64;
    MatchAllKeyword : int64;
  end;
  PTRACE_ENABLE_INFO = ^TRACE_ENABLE_INFO;

//
// Instance Information for Provider 
// Used on Vista and above
//
  TRACE_PROVIDER_INSTANCE_INFO = packed record
    NextOffset : cardinal;
    EnableCount : cardinal;
    Pid : cardinal;
    Flags : cardinal;
  end;
  PTRACE_PROVIDER_INSTANCE_INFO = ^TRACE_PROVIDER_INSTANCE_INFO;

//
// GUID Information Used on Vista and above
//
  TRACE_GUID_INFO = packed record
    InstanceCount : cardinal;
    Reserved : cardinal;
  end;
  PTRACE_GUID_INFO = ^TRACE_GUID_INFO;

//
// An EVENT_TRACE consists of a fixed header (EVENT_TRACE_HEADER) and
// optionally a variable portion pointed to by MofData. The datablock
// layout of the variable portion is unknown to the Logger and must
// be obtained from WBEM CIMOM database.
//
  EVENT_TRACE = packed record
    Header : EVENT_TRACE_HEADER;             // Event trace header
    InstanceId : cardinal;         // Instance Id of this event
    ParentInstanceId : cardinal;   // Parent Instance Id.
    ParentGuid : TGUID;         // Parent Guid;
    MofData : pointer;            // Pointer to Variable Data
    MofLength : cardinal;          // Variable Datablock Length
    ClientContext : cardinal;
  end;
  PEVENT_TRACE = ^EVENT_TRACE;

  EVENT_RECORD = packed record
  end;
  PEVENT_RECORD = ^EVENT_RECORD;

  EVENT_TRACE_LOGFILEW = packed record
  end;
  PEVENT_TRACE_LOGFILEW = ^EVENT_TRACE_LOGFILEW;

  EVENT_TRACE_LOGFILEA = packed record
  end;
  PEVENT_TRACE_LOGFILEA = ^EVENT_TRACE_LOGFILEA;

  PEVENT_TRACE_BUFFER_CALLBACKW = function (Logfile : PEVENT_TRACE_LOGFILEW) : cardinal; stdcall;
  PEVENT_TRACE_BUFFER_CALLBACKA = function (Logfile : PEVENT_TRACE_LOGFILEA) : cardinal; stdcall;  
  PEVENT_CALLBACK = procedure(pEvent : PEVENT_TRACE); stdcall;
  PEVENT_RECORD_CALLBACK = procedure(EventRecord : PEVENT_RECORD); stdcall;

//
// Prototype for service request callback. Data providers register with WMI
// by passing a service request callback function that is called for all
// wmi requests.
  WMIDPREQUESTCODE = cardinal;
  WMIDPREQUEST = function(RequestCode : WMIDPREQUESTCODE;
                          RequestContext : pointer;
                          var BufferSize : cardinal;
                          Buffer : pointer) : cardinal;

  _EVENT_TRACE_LOGFILEW = packed record
    LogFileName : PWideChar;                 // Logfile Name
    LoggerName : PWideChar;                  // LoggerName
    CurrentTime : int64;                     // timestamp of last event
    BuffersRead : cardinal;                  // buffers read to date
    LogFileMode : cardinal;
    CurrentEvent : EVENT_TRACE;              // Current Event from this stream.
    LogfileHeader : TRACE_LOGFILE_HEADER;            // logfile header structure
    BufferCallback : PEVENT_TRACE_BUFFER_CALLBACKW;  // callback before each buffer is read
    //
    // following variables are filled for BufferCallback.
    //
    BufferSize : cardinal;
    Filled : cardinal;
    EventsLost : cardinal;
    //
    // following needs to be propaged to each buffer
    //
    EventCallback : PEVENT_CALLBACK;
    IsKernelTrace : cardinal;    // TRUE for kernel logfile
    Context : pointer;          // reserved for internal use
  end;

   _EVENT_TRACE_LOGFILE = record
     LogFileName : LPSTR;
     LoggerName : LPSTR;
//     filler : int64;
     CurrentTime : LONGLONG;
     BuffersRead : ULONG;
     LogFileMode : ULONG;
     CurrentEvent : EVENT_TRACE;
     filler :cardinal;
     LogfileHeader : TRACE_LOGFILE_HEADER;
     filler2 : cardinal;
     BufferCallback : PEVENT_TRACE_BUFFER_CALLBACKA;
     BufferSize : cardinal;
     Filled : cardinal;
     EventsLost : cardinal;
     EventCallback : PEVENT_CALLBACK;
     IsKernelTrace : cardinal;
     Context : pointer;
   end;

  _EVENT_TRACE_LOGFILEA = packed record
    LogFileName : PChar;                 // Logfile Name
    LoggerName : PChar;                  // LoggerName
    CurrentTime : int64;                     // timestamp of last event
    BuffersRead : cardinal;                  // buffers read to date
    LogFileMode : cardinal;
    CurrentEvent : EVENT_TRACE;              // Current Event from this stream.
    LogfileHeader : TRACE_LOGFILE_HEADER;            // logfile header structure
    BufferCallback : PEVENT_TRACE_BUFFER_CALLBACKA;  // callback before each buffer is read
    //
    // following variables are filled for BufferCallback.
    //
    BufferSize : cardinal;
    Filled : cardinal;
    EventsLost : cardinal;
    //
    // following needs to be propaged to each buffer
    //
    EventCallback : PEVENT_CALLBACK;
    IsKernelTrace : cardinal;    // TRUE for kernel logfile
    Context : pointer;          // reserved for internal use
  end;

  MOF_FIELD = packed record
    DataPtr : pointer;
    reserved : cardinal;
    Length : cardinal;
    DataType : cardinal;
  end;

//
// Define generic structures
//
type
  PEVENT_TRACE_BUFFER_CALLBACK  = PEVENT_TRACE_BUFFER_CALLBACKA;
  EVENT_TRACE_LOGFILE           = _EVENT_TRACE_LOGFILE;
  PEVENT_TRACE_LOGFILE          = PEVENT_TRACE_LOGFILEA;


  EVENT_FILTER_DESCRIPTOR = packed record
  end;
  PEVENT_FILTER_DESCRIPTOR = ^EVENT_FILTER_DESCRIPTOR;

  TRACE_QUERY_INFO_CLASS = (TraceGuidQueryList, TraceGuidQueryInfo, TraceGuidQueryProcess);


function StartTrace(var TraceHandle : TRACEHANDLE; InstanceName : PChar; Properties : PEVENT_TRACE_PROPERTIES) : cardinal;
function StopTrace(TraceHandle : TRACEHANDLE; InstanceName : PChar; Properties : PEVENT_TRACE_PROPERTIES) : cardinal;
function QueryTrace(TraceHandle : TRACEHANDLE; InstanceName : PChar; Properties : PEVENT_TRACE_PROPERTIES) : cardinal;
function UpdateTrace(TraceHandle : TRACEHANDLE; InstanceName : PChar; Properties : PEVENT_TRACE_PROPERTIES) : cardinal;
function FlushTrace(TraceHandle : TRACEHANDLE; InstanceName : PChar; Properties : PEVENT_TRACE_PROPERTIES) : cardinal;
function ControlTrace(TraceHandle : TRACEHANDLE; InstanceName : PWideChar; Properties : PEVENT_TRACE_PROPERTIES; ControlCode : cardinal) : cardinal;
function QueryAllTraces(PropertyArray : pointer; PropertyArrayCount : cardinal; var LoggerCount : cardinal) : cardinal;
function EnableTrace(Enable : cardinal; EnableFlag : cardinal; EnableLevel : cardinal; ControlGuid : PGUID; TraceHandle : TRACEHANDLE) : cardinal;
function EnableTraceEx(ProviderId : PGUID;
                       SourceId : PGUID;
                       TraceHandle : TRACEHANDLE;
                       IsEnabled : boolean;
                       Level : byte;
                       MatchAnyKeyword : int64;
                       MatchAllKeyword : int64;
                       EnableProperty : cardinal;
                       EnableFilterDesc : PEVENT_FILTER_DESCRIPTOR) : cardinal;
function EnumerateTraceGuidsEx(TraceQueryInfoClass : TRACE_QUERY_INFO_CLASS;
                               InBuffer : pointer; InBufferSize : cardinal;
                               OutBuffer : pointer; OutBufferSize : cardinal;
                               var ReturnLength : cardinal) : cardinal;
function CreateTraceInstanceId(RegHandle : handle; pInstInfo : PEVENT_INSTANCE_INFO) : cardinal;
function TraceEvent(TraceHandle : TRACEHANDLE; EventTrace : PEVENT_TRACE_HEADER) : cardinal;
function TraceEventInstance(TraceHandle : TRACEHANDLE; EventTrace : PEVENT_INSTANCE_HEADER; pInstInfo : PEVENT_INSTANCE_INFO; pParentInstInfo : PEVENT_INSTANCE_INFO) : cardinal;


function RegisterTraceGuids(RequestAddress : WMIDPREQUEST; RequestContext : pointer;
                            ControlGuid : PGUID; GuidCount : cardinal;
                            TraceGuidReg : PTRACE_GUID_REGISTRATION;
                            MofImagePath : PChar;
                            MofResourceName : PChar;
                            var RegistrationHandle : TRACEHANDLE) : cardinal;
function EnumerateTraceGuids (GuidPropertiesArray : pointer;
                              PropertyArrayCount : cardinal;
                              var GuidCount : cardinal): cardinal;
function UnregisterTraceGuids(RegistrationHandle : TRACEHANDLE): cardinal;
function GetTraceLoggerHandle(Buffer : pointer) : TRACEHANDLE;
function GetTraceEnableLevel(TraceHandle : TRACEHANDLE) : byte;
function GetTraceEnableFlags(TraceHandle : TRACEHANDLE) : cardinal;
function OpenTrace(Logfile : PEVENT_TRACE_LOGFILEA) : TRACEHANDLE;
function ProcessTrace(HandleArray : PTRACEHANDLE; HandleCount : cardinal; StartTime : pointer; EndTime : pointer) : cardinal;
function CloseTrace(TraceHandle : TRACEHANDLE) : cardinal;
function SetTraceCallback(pGuid : PGUID; EventCallback : PEVENT_CALLBACK) : cardinal;
function RemoveTraceCallback(pGuid : PGUID) : cardinal;

implementation
var
  hModule : cardinal;

type
//
// Logger control APIs
//

//
// Use the routine below to start an event trace session
//

// ULONG
// StartTrace(
//      OUT PTRACEHANDLE TraceHandle,
//      IN LPTSTR InstanceName,
//      IN OUT PEVENT_TRACE_PROPERTIES Properties
//      );

  TStartTraceW = function(var TraceHandle : TRACEHANDLE;
                          InstanceName : PWideChar;
                          Properties : PEVENT_TRACE_PROPERTIES) : cardinal; stdcall;
  TStartTraceA = function(var TraceHandle : TRACEHANDLE;
                          InstanceName : PChar;
                          Properties : PEVENT_TRACE_PROPERTIES) : cardinal; stdcall;

//
// Use the routine below to stop an event trace session
//

//
// ULONG
// StopTrace(
//      IN TRACEHANDLE TraceHandle,
//      IN LPTSTR InstanceName,
//      IN OUT PEVENT_TRACE_PROPERTIES Properties
//      );

  TStopTraceW = function(TraceHandle : TRACEHANDLE; InstanceName : PChar;
                         Properties : PEVENT_TRACE_PROPERTIES) : cardinal; stdcall;
  TStopTraceA = function(TraceHandle : TRACEHANDLE; InstanceName : PChar;
                         Properties : PEVENT_TRACE_PROPERTIES) : cardinal; stdcall;
//
// Use the routine below to query the properties of an event trace session
//

// ULONG
// QueryTrace(
//      IN TRACEHANDLE TraceHandle,
//      IN LPTSTR InstanceName,
//      IN OUT PEVENT_TRACE_PROPERTIES Properties
//      );
  TQueryTraceW=function(TraceHandle : TRACEHANDLE; InstanceName : PWideChar; Properties : PEVENT_TRACE_PROPERTIES) : cardinal; stdcall;
  TQueryTraceA=function(TraceHandle : TRACEHANDLE; InstanceName : PChar; Properties : PEVENT_TRACE_PROPERTIES) : cardinal; stdcall;

//
// Use the routine below to update certain properties of an event trace session
//

// ULONG
// UpdateTrace(
//      IN (PTRACEHANDLE TraceHandle,
//      IN LPTSTR InstanceName,
//      IN OUT PEVENT_TRACE_PROPERTIES Properties
//      );
  TUpdateTraceW=function(TraceHandle : TRACEHANDLE; InstanceName : PWideChar; Properties : PEVENT_TRACE_PROPERTIES) : cardinal; stdcall;
  TUpdateTraceA=function(TraceHandle : TRACEHANDLE; InstanceName : PChar; Properties : PEVENT_TRACE_PROPERTIES) : cardinal; stdcall;

//
// Use the routine below to request that all active buffers an event trace
// session be "flushed", or written out.
//

// ULONG
// FlushTrace(
//      IN TRACEHANDLE TraceHandle,
//      IN LPTSTR InstanceName,
//      IN OUT PEVENT_TRACE_PROPERTIES Properties
//      );
  TFlushTraceW=function(TraceHandle : TRACEHANDLE; InstanceName : PWideChar; Properties : PEVENT_TRACE_PROPERTIES) : cardinal; stdcall;
  TFlushTraceA=function(TraceHandle : TRACEHANDLE; InstanceName : PChar; Properties : PEVENT_TRACE_PROPERTIES) : cardinal; stdcall;

//
// Generic trace control routine
//
  TControlTraceW = function(TraceHandle : TRACEHANDLE; InstanceName : PWideChar; Properties : PEVENT_TRACE_PROPERTIES; ControlCode : cardinal) : cardinal; stdcall;
  TControlTraceA = function(TraceHandle : TRACEHANDLE; InstanceName : PWideChar; Properties : PEVENT_TRACE_PROPERTIES; ControlCode : cardinal) : cardinal; stdcall;

  
//
// ULONG
// QueryAllTraces(
//  OUT PEVENT_TRACE_PROPERTIES *PropertyArray,
//  IN ULONG PropertyArrayCount,
//  OUT PULONG LoggerCount
//  );
//
  TQueryAllTracesW = function(PropertyArray : pointer; PropertyArrayCount : cardinal; var LoggerCount : cardinal) : cardinal; stdcall;
  TQueryAllTracesA = function(PropertyArray : pointer; PropertyArrayCount : cardinal; var LoggerCount : cardinal) : cardinal; stdcall;


//
// Data Provider Enable APIs
//
  TEnableTrace=function(Enable : cardinal; EnableFlag : cardinal; EnableLevel : cardinal; ControlGuid : PGUID; TraceHandle : TRACEHANDLE) : cardinal; stdcall;
  TEnumerateTraceGuidsEx = function(TraceQueryInfoClass : TRACE_QUERY_INFO_CLASS;
                                    InBuffer : pointer; InBufferSize : cardinal;
                                    OutBuffer : pointer; OutBufferSize : cardinal;
                                    var ReturnLength : cardinal) : cardinal; stdcall;

//
// Data Provider APIs
//
  TCreateTraceInstanceId = function(RegHandle : handle; pInstInfo : PEVENT_INSTANCE_INFO) : cardinal; stdcall;

//
// Use the routine below to generate and record an event trace
//
  TTraceEvent = function(TraceHandle : TRACEHANDLE; EventTrace : PEVENT_TRACE_HEADER) : cardinal; stdcall;
  TTraceEventInstance = function(TraceHandle : TRACEHANDLE; EventTrace : PEVENT_INSTANCE_HEADER; pInstInfo : PEVENT_INSTANCE_INFO; pParentInstInfo : PEVENT_INSTANCE_INFO) : cardinal; stdcall;

//
// Use the routine below to register a guid for tracing
//

//
// ULONG
// RegisterTraceGuids(
//  IN WMIDPREQUEST  RequestAddress,
//  IN PVOID         RequestContext,
//  IN LPCGUID       ControlGuid,
//  IN ULONG         GuidCount,
//  IN PTRACE_GUID_REGISTRATION TraceGuidReg,
//  IN LPCTSTR       MofImagePath,
//  IN LPCTSTR       MofResourceName,
//  OUT PTRACEHANDLE RegistrationHandle
//  );
//
  TRegisterTraceGuidsW = function(RequestAddress : WMIDPREQUEST; RequestContext : pointer;
                                  ControlGuid : PGUID; GuidCount : cardinal;
                                  TraceGuidReg : PTRACE_GUID_REGISTRATION;
                                  MofImagePath : PWideChar;
                                  MofResourceName : PWideChar;
                                  var RegistrationHandle : TRACEHANDLE) : cardinal; stdcall;
  TRegisterTraceGuidsA = function(RequestAddress : WMIDPREQUEST; RequestContext : pointer;
                                  ControlGuid : PGUID; GuidCount : cardinal;
                                  TraceGuidReg : PTRACE_GUID_REGISTRATION;
                                  MofImagePath : PChar;
                                  MofResourceName : PChar;
                                  var RegistrationHandle : TRACEHANDLE) : cardinal; stdcall;

  TEnumerateTraceGuids = function(GuidPropertiesArray : pointer;
                                  PropertyArrayCount : cardinal;
                                  var GuidCount : cardinal): cardinal; stdcall;
  TUnregisterTraceGuids = function(RegistrationHandle : TRACEHANDLE): cardinal; stdcall;
  TGetTraceLoggerHandle = function(Buffer : pointer) : TRACEHANDLE; stdcall;
  TGetTraceEnableLevel = function(TraceHandle : TRACEHANDLE) : byte; stdcall;
  TGetTraceEnableFlags = function(TraceHandle : TRACEHANDLE) : cardinal; stdcall;

//
// Data Consumer APIs and structures start here
//

//
// TRACEHANDLE
// OpenTrace(
//  IN OUT PEVENT_TRACE_LOGFILE Logfile
//  );
//
  TOpenTraceA=function(Logfile : PEVENT_TRACE_LOGFILEA) : TRACEHANDLE; stdcall;
  TOpenTraceW=function(Logfile : PEVENT_TRACE_LOGFILEW) : TRACEHANDLE; stdcall;
  TProcessTrace=function(HandleArray : PTRACEHANDLE; HandleCount : cardinal; StartTime : pointer; EndTime : pointer) : cardinal; stdcall;
  TCloseTrace=function(TraceHandle : TRACEHANDLE) : cardinal; stdcall;
  TSetTraceCallback = function(pGuid : PGUID; EventCallback : PEVENT_CALLBACK) : cardinal; stdcall;
  TRemoveTraceCallback = function(pGuid : PGUID) : cardinal; stdcall;


//
// The routines for tracing Messages follow
//

{
EXTERN_C
ULONG
__cdecl
TraceMessage(
    __in TRACEHANDLE  LoggerHandle,
    __in ULONG        MessageFlags,
    __in LPGUID       MessageGuid,
    __in USHORT       MessageNumber,
    ...
);

EXTERN_C
ULONG
TraceMessageVa(
    __in TRACEHANDLE  LoggerHandle,
    __in ULONG        MessageFlags,
    __in LPGUID       MessageGuid,
    __in USHORT       MessageNumber,
    __in va_list      MessageArgList
);
}
  TEnableTraceEx = function(ProviderId : PGUID;
                            SourceId : PGUID;
                            TraceHandle : TRACEHANDLE;
                            IsEnabled : boolean;
                            Level : byte;
                            MatchAnyKeyword : int64;
                            MatchAllKeyword : int64;
                            EnableProperty : cardinal;
                            EnableFilterDesc : PEVENT_FILTER_DESCRIPTOR) : cardinal; stdcall;
var
  StartTraceProc : TStartTraceA = nil;
  StopTraceProc : TStopTraceA = nil;
  QueryTraceProc : TQueryTraceA = nil;
  UpdateTraceProc : TUpdateTraceA = nil;
  FlushTraceProc : TFlushTraceA = nil;
  ControlTraceProc : TControlTraceA = nil;
  QueryAllTracesProc : TQueryAllTracesA = nil;
  EnableTraceProc : TEnableTrace = nil;
  EnableTraceExProc : TEnableTraceEx = nil;
  EnumerateTraceGuidsExProc : TEnumerateTraceGuidsEx = nil;
  CreateTraceInstanceIdProc : TCreateTraceInstanceId = nil;
  TraceEventProc : TTraceEvent = nil;
  TraceEventInstanceProc : TTraceEventInstance = nil;
  RegisterTraceGuidsProc : TRegisterTraceGuidsA = nil;
  EnumerateTraceGuidsProc : TEnumerateTraceGuids = nil;
  UnregisterTraceGuidsProc : TUnregisterTraceGuids = nil;
  GetTraceLoggerHandleProc : TGetTraceLoggerHandle = nil;
  GetTraceEnableLevelProc : TGetTraceEnableLevel = nil;
  GetTraceEnableFlagsProc : TGetTraceEnableFlags = nil;
  OpenTraceProc : TOpenTraceA = nil;
  ProcessTraceProc : TProcessTrace = nil;
  CloseTraceProc : TCloseTrace = nil;
  SetTraceCallbackProc : TSetTraceCallback = nil;
  RemoveTraceCallbackProc : TRemoveTraceCallback = nil;

function StartTrace(var TraceHandle : TRACEHANDLE; InstanceName : PChar; Properties : PEVENT_TRACE_PROPERTIES) : cardinal;
begin
  if Assigned(@StartTraceProc)
  then result := StartTraceProc(TraceHandle, InstanceName, Properties)
  else result := 0;
end;

function StopTrace(TraceHandle : TRACEHANDLE; InstanceName : PChar; Properties : PEVENT_TRACE_PROPERTIES) : cardinal;
begin
  if Assigned(@StopTraceProc)
  then result := StopTraceProc(TraceHandle, InstanceName, Properties)
  else result := 0;
end;

function QueryTrace(TraceHandle : TRACEHANDLE; InstanceName : PChar; Properties : PEVENT_TRACE_PROPERTIES) : cardinal;
begin
  if Assigned(@QueryTraceProc)
  then result := QueryTraceProc(TraceHandle, InstanceName, Properties)
  else result := 0;
end;

function UpdateTrace(TraceHandle : TRACEHANDLE; InstanceName : PChar; Properties : PEVENT_TRACE_PROPERTIES) : cardinal;
begin
  if Assigned(@UpdateTraceProc)
  then result := UpdateTraceProc(TraceHandle, InstanceName, Properties)
  else result := 0;
end;

function FlushTrace(TraceHandle : TRACEHANDLE; InstanceName : PChar; Properties : PEVENT_TRACE_PROPERTIES) : cardinal;
begin
  if Assigned(@FlushTraceProc)
  then result := UpdateTraceProc(TraceHandle, InstanceName, Properties)
  else result := 0;
end;

function ControlTrace(TraceHandle : TRACEHANDLE; InstanceName : PWideChar; Properties : PEVENT_TRACE_PROPERTIES; ControlCode : cardinal) : cardinal;
begin
  if Assigned(@ControlTraceProc)
  then result := ControlTraceProc(TraceHandle, InstanceName, Properties, ControlCode)
  else result := 0;
end;

function QueryAllTraces(PropertyArray : pointer; PropertyArrayCount : cardinal; var LoggerCount : cardinal) : cardinal;
begin
  if Assigned(@QueryAllTracesProc)
  then result := QueryAllTracesProc(PropertyArray, PropertyArrayCount, LoggerCount)
  else result := 0;
end;

function EnableTrace(Enable : cardinal; EnableFlag : cardinal; EnableLevel : cardinal; ControlGuid : PGUID; TraceHandle : TRACEHANDLE) : cardinal;
begin
  if Assigned(@EnableTraceProc)
  then result := EnableTraceProc(Enable, EnableFlag, EnableLevel, ControlGuid, TraceHandle)
  else result := 0;
end;

function EnableTraceEx(ProviderId : PGUID;
                       SourceId : PGUID;
                       TraceHandle : TRACEHANDLE;
                       IsEnabled : boolean;
                       Level : byte;
                       MatchAnyKeyword : int64;
                       MatchAllKeyword : int64;
                       EnableProperty : cardinal;
                       EnableFilterDesc : PEVENT_FILTER_DESCRIPTOR) : cardinal;
begin
  if Assigned(@EnableTraceExProc)
  then result := EnableTraceExProc(ProviderId, SourceId, TraceHandle,
                    IsEnabled, Level, MatchAnyKeyword, MatchAllKeyword, EnableProperty, EnableFilterDesc)
  else result := 0;
end;

function EnumerateTraceGuidsEx(TraceQueryInfoClass : TRACE_QUERY_INFO_CLASS;
                               InBuffer : pointer; InBufferSize : cardinal;
                               OutBuffer : pointer; OutBufferSize : cardinal;
                               var ReturnLength : cardinal) : cardinal;
begin
  if Assigned(@EnumerateTraceGuidsExProc)
  then result := EnumerateTraceGuidsExProc(TraceQueryInfoClass, InBuffer, InBufferSize,
                    OutBuffer, OutBufferSize, ReturnLength)
  else result := 0;
end;

function CreateTraceInstanceId(RegHandle : handle; pInstInfo : PEVENT_INSTANCE_INFO) : cardinal;
begin
  if Assigned(@CreateTraceInstanceIdProc)
  then result := CreateTraceInstanceIdProc(RegHandle, pInstInfo)
  else result := 0;
end;

function TraceEvent(TraceHandle : TRACEHANDLE; EventTrace : PEVENT_TRACE_HEADER) : cardinal;
begin
  if Assigned(@TraceEventProc)
  then result := TraceEventProc(TraceHandle, EventTrace)
  else result := 0;
end;

function TraceEventInstance(TraceHandle : TRACEHANDLE; EventTrace : PEVENT_INSTANCE_HEADER;
      pInstInfo : PEVENT_INSTANCE_INFO; pParentInstInfo : PEVENT_INSTANCE_INFO) : cardinal;
begin
  if Assigned(@TraceEventInstanceProc)
  then result := TraceEventInstanceProc(TraceHandle, EventTrace, pInstInfo, pParentInstInfo)
  else result := 0;
end;

function RegisterTraceGuids(RequestAddress : WMIDPREQUEST; RequestContext : pointer;
                            ControlGuid : PGUID; GuidCount : cardinal;
                            TraceGuidReg : PTRACE_GUID_REGISTRATION;
                            MofImagePath : PChar;
                            MofResourceName : PChar;
                            var RegistrationHandle : TRACEHANDLE) : cardinal;
begin
  if Assigned(@RegisterTraceGuidsProc)
  then result := RegisterTraceGuidsProc(RequestAddress, RequestContext, ControlGuid,
                      GuidCount,TraceGuidReg,  MofImagePath, MofResourceName, RegistrationHandle)
  else result := ERROR_SUCCESS;
end;

function EnumerateTraceGuids (GuidPropertiesArray : pointer;
                              PropertyArrayCount : cardinal;
                              var GuidCount : cardinal): cardinal;
begin
  if Assigned(@EnumerateTraceGuidsProc)
  then result := EnumerateTraceGuidsProc(GuidPropertiesArray, PropertyArrayCount, GuidCount)
  else result := 0;
end;

function UnregisterTraceGuids(RegistrationHandle : TRACEHANDLE): cardinal;
begin
  if Assigned(@UnregisterTraceGuidsProc)
  then result := UnregisterTraceGuidsProc(RegistrationHandle)
  else result := 0;
end;

function GetTraceLoggerHandle(Buffer : pointer) : TRACEHANDLE;
begin
  if Assigned(@GetTraceLoggerHandleProc)
  then result := GetTraceLoggerHandleProc(Buffer)
  else result := INVALID_HANDLE_VALUE;
end;

function GetTraceEnableLevel(TraceHandle : TRACEHANDLE) : byte;
begin
  if Assigned(@GetTraceEnableLevelProc)
  then result := GetTraceEnableLevelProc(TraceHandle)
  else result := 0;
end;

function GetTraceEnableFlags(TraceHandle : TRACEHANDLE) : cardinal;
begin
  if Assigned(@GetTraceEnableFlagsProc)
  then result := GetTraceEnableFlagsProc(TraceHandle)
  else result := 0;
end;

function OpenTrace(Logfile : PEVENT_TRACE_LOGFILEA) : TRACEHANDLE;
begin
  if Assigned(@OpenTraceProc)
  then result := OpenTraceProc(Logfile)
  else result := 0;
end;

function ProcessTrace(HandleArray : PTRACEHANDLE; HandleCount : cardinal; StartTime : pointer; EndTime : pointer) : cardinal;
begin
  if Assigned(@ProcessTraceProc)
  then result := ProcessTraceProc(HandleArray, HandleCount, StartTime, EndTime)
  else result := 0;
end;


function CloseTrace(TraceHandle : TRACEHANDLE) : cardinal;
begin
  if Assigned(@CloseTraceProc)
  then result := CloseTraceProc(TraceHandle)
  else result := 0;
end;

function SetTraceCallback(pGuid : PGUID; EventCallback : PEVENT_CALLBACK) : cardinal;
begin
  if Assigned(@SetTraceCallbackProc)
  then result := SetTraceCallbackProc(pGuid, EventCallback)
  else result := 0;
end;

function RemoveTraceCallback(pGuid : PGUID) : cardinal;
begin
  if Assigned(@RemoveTraceCallbackProc)
  then result := RemoveTraceCallbackProc(pGuid)
  else result := 0;
end;

procedure LoadAdvApi32;
begin
  hModule := LoadLibrary('ADVAPI32.DLL');
  if hModule<>INVALID_HANDLE_VALUE
  then begin
    @StartTraceProc := GetProcAddress(hModule, 'StartTraceA');
    @StopTraceProc := GetProcAddress(hModule, 'StopTraceA');
    @QueryTraceProc := GetProcAddress(hModule, 'QueryTraceA');
    @UpdateTraceProc := GetProcAddress(hModule, 'UpdateTraceA');
    @FlushTraceProc := GetProcAddress(hModule, 'FlushTraceA');
    @ControlTraceProc := GetProcAddress(hModule, 'ControlTraceA');
    @QueryAllTracesProc := GetProcAddress(hModule, 'QueryAllTracesA');
    @EnableTraceProc := GetProcAddress(hModule, 'EnableTrace');
    @EnableTraceExProc := GetProcAddress(hModule, 'EnableTraceEx');
    @EnumerateTraceGuidsExProc := GetProcAddress(hModule, 'EnumerateTraceGuidsEx');
    @CreateTraceInstanceIdProc := GetProcAddress(hModule, 'CreateTraceInstanceId');
    @TraceEventProc := GetProcAddress(hModule, 'TraceEvent');
    @TraceEventInstanceProc := GetProcAddress(hModule, 'TraceEventInstance');
    @RegisterTraceGuidsProc := GetProcAddress(hModule, 'RegisterTraceGuidsA');
    @EnumerateTraceGuidsProc := GetProcAddress(hModule, 'EnumerateTraceGuids');
    @UnregisterTraceGuidsProc := GetProcAddress(hModule, 'UnregisterTraceGuids');
    @GetTraceLoggerHandleProc := GetProcAddress(hModule, 'GetTraceLoggerHandle');
    @GetTraceEnableLevelProc := GetProcAddress(hModule, 'GetTraceEnableLevel');
    @GetTraceEnableFlagsProc := GetProcAddress(hModule, 'GetTraceEnableFlags');
    @OpenTraceProc := GetProcAddress(hModule, 'OpenTraceA');
    @ProcessTraceProc := GetProcAddress(hModule, 'ProcessTrace');
    @CloseTraceProc := GetProcAddress(hModule, 'CloseTrace');
    @SetTraceCallbackProc := GetProcAddress(hModule, 'SetTraceCallback');
    @RemoveTraceCallbackProc := GetProcAddress(hModule, 'RemoveTraceCallback');
  end;
end;

procedure UnloadAdvApi32;
begin
  FreeLibrary(hModule);
  @StartTraceProc := nil;
  @StopTraceProc := nil;
  @QueryTraceProc := nil;
  @UpdateTraceProc := nil;
  @FlushTraceProc := nil;
  @ControlTraceProc := nil;
  @QueryAllTracesProc := nil;
  @EnableTraceProc := nil;
  @EnableTraceExProc := nil;
  @EnumerateTraceGuidsExProc := nil;
  @CreateTraceInstanceIdProc := nil;
  @TraceEventProc := nil;
  @TraceEventInstanceProc := nil;
  @RegisterTraceGuidsProc := nil;
  @EnumerateTraceGuidsProc := nil;
  @UnregisterTraceGuidsProc := nil;
  @GetTraceLoggerHandleProc := nil;
  @GetTraceEnableLevelProc := nil;
  @GetTraceEnableFlagsProc := nil;
  @OpenTraceProc := nil;
  @ProcessTraceProc := nil;
  @CloseTraceProc := nil;
  @SetTraceCallbackProc := nil;
  @RemoveTraceCallbackProc := nil;
end;

initialization
  LoadAdvApi32;
finalization
  UnloadAdvApi32;
end.
